<?php


class Classification {
	
	private $uid;

	private $id;
	private $value;
	private $static;
	private $anonymous;
	
	private $parent;
		
	// Array that hold all values
	public $classificationValues;
	
	// User input
	private $useAnonymous;
	private $selected;
	private $selectedClassificationValues;	// Could be removed?
	private $loading;
	
	// Relations
	private $relations;
		
	// Virtual GUI Classification
	private $virtualGui;
	
	// Variables used by Templates
	public $max;
	public $min;
	public $restricted;
	public $allowInput;
	
	
	public function __construct($uid, $id, $value, $static, $anonymous, DataType $parent, $virtualGui) {
		
		// Don’t load values, we do that on request only
		$this->uid = $uid;
		$this->id = $id;
		$this->value = $value;
		$this->static = $static;
		$this->anonymous = $anonymous;
		
		$this->virtualGui = $virtualGui;
				
		$this->parent = $parent;
		
		$this->selected = false;
		$this->selectedClassificationValues = array();
		
		$this->loading = false;
		
		// Check if the user is allowd to see the real values, or if it should see the "masked" value
		$this->useAnonymous = $_SESSION["USER"]->isClassificationAnonymous($parent->getId(), $this->id);
		
		// Default for Templates values
		$this->max = -1;
		$this->min = -1;
		$this->restricted = array();
		$this->allowInput = false;
	}
	
	public function __destruct() {
		// Make sure that we don’t put this list in memory, it’s too big
		$this->classificationValues = array();
	}
	
	
	// Normal
	public function getUID() {
		return $this->uid;
	}
	
	public function getID() {
		return $this->id;
	}
	
	public function getValue() {
		return $this->value;
	}
	
	public function isStatic()  {
		return $this->static ? true : false;
	}
	
	
	public function isAnonymous() {
		return $this->anonymous ? true : false;
	}
	
	public function isUseAnonymous() {
		return $this->useAnonymous ? true : false;
	}
	
	public function isSelectionValid() {
		$selections = count(ClassificationSelectionManager::getClassificationValueSelection($this->uid));
		$max = $this->max;
		$min = $this->min;
		
		return (($max != -1 && $selections > $max) || ($selections < $min)) ? false : true;
	}
	
	
	/**
	 * Returns parent DataType
	 *
	 * @return DataType
	 */
	public function getDataType() {
		return $this->parent;
	}
	
	
	// For testing
	public function setUseAnonymous($anon) {
		if ($this->anonymous) {
			$this->useAnonymous = $anon ? true : false;
		}
	}
	
	
	
	public function isSelected() {
		return $this->selected ? true : false;
	}

	public function setSelected($boolean) {
		$this->selected = $boolean ? true : false;
	}
	
	public function isAllowInput() {
		return $this->allowInput ? true : false;
	}
	
	// Advanced
	
	
	public function getClassificationValues($depricated_not_in_use = true) {

		// Don't try to fetch the classification values from 
		if ($this->id == 'T') return array();
		
		$allowedValues = null;
				
		if (isset($this->restricted) && is_array($this->restricted) && !empty($this->restricted)) {
			$allowedValues = array();
			foreach ($this->restricted as $value) {
				$allowedValues[$value] = 1;
			}
			Logger::Debug("Classification::getClassificationValues()", "Restrictions is set!");
		}
		
		// NEW
		return ClassificationValue::sortList($this->classificationValues = $this->filter(DatabaseManager::getValuesNew2($this), $this->getValue()));
		/*
		if ($this->loading) {
			Logger::Debug("Classification::getClassificationValues()" , "ALLREADY LOADING");
			return array();
		}
			
		$this->loading = true;
		
		if (empty($this->classificationValues) && !empty($this->selectedClassificationValues) && !$deep) {
			$this->loading = false;
			return ClassificationValue::sortList($this->filter(DatabaseManager::getClassificationValuesFromSelection($this)));
		}

		
		if (empty($this->classificationValues) && empty($classificationValues)) {
			$this->classificationValues = $this->filter(DatabaseManager::getValuesNew($this));
		}

		$this->loading = false;
		return ClassificationValue::sortList($this->classificationValues);
		*/
	}
	
	private function filter($classificationValues, $clsfParent) {
		$prefilteredValues = array();
		Logger::Debug(__CLASS__.'::'.__METHOD__, 'Processing classification value access...');
		foreach ($classificationValues as $value) {
			if (!$_SESSION["USER"]->hasValueFiltered($clsfParent, $value->getValue(false))) {
				$prefilteredValues[] = $value;
			}
		}
		
		//Logger::Debug("prefilteredvalues", count($prefilteredValues));
		if (!empty($this->restricted) && is_array($this->restricted)) {
			Logger::Debug(__CLASS__.'::'.__METHOD__, 'Running filtering...');
			$allowedValues = array();
			foreach ($this->restricted as $value) {
				$allowedValues[$value] = 1;
			}

			$values = array();
			foreach ($prefilteredValues as $value) {
				if (isset($allowedValues[$value->getValue(false)])) {
					$values[] = $value;
				}
			}
			//
			return $values;
		}
		Logger::Debug(__METHOD__, 'Return without any filtering...');
		return $prefilteredValues;
	}
	
	
	public function markClassificationValueAsSelected(ClassificationValue $classificationvalue) {
		$this->selectedClassificationValues[] = $classificationvalue->getUID();
	}
	
	
	public function markAllClassificationValuesAsSelected() {
		$classificationvalues = $this->getClassificationValues(false);
		
		// TODO: Its a big mix of the old selection style, and the new.. I would need to clean up this..
		ClassificationSelectionManager::removeAllClassificationValueSelection($this->getUID());

		foreach ($classificationvalues as $classificationvalue) {
			$this->selectedClassificationValues[] = $classificationvalue->getUID();
			ClassificationSelectionManager::addClassificationValueSelection($this->getUID(), $classificationvalue);
		}		
				
	}
	
	

	public function addSelection($uid) {
		$this->selectedClassificationValues[] = $uid;
	}

	public function getMarkedClassificationValues() {
		return $this->selectedClassificationValues;
	}
	
	public function removeSelections() {
		$this->selectedClassificationValues = array();
	}
	
	public function hasValueSelected() {
		return count($this->selectedClassificationValues) >= 1 ? true : false;
	}
	
	public function getRelationIds() {
		return DatabaseManager::getRelations($this);
	}
	
	public function getSelectedClassificationValues() {
		return DatabaseManager::getClassificationValuesFromSelection($this);
	}
	
	public function isVirtualGui() {
		return $this->virtualGui;
	}
	
	
}

?>