<?php


class DataType {

	private $uid;
	
	private $id;
	private $value;
	private $moduleId;
	
	private $dataFrom;
	private $dataTo;
	
	// Array that holds classification
	private $classifications;	
	private $measurements;
	
	public $reportData;

	function __construct($uid, $id, $value, $dataFrom, $dataTo, $moduleId = null) {
		
		$this->uid = $uid;
		$this->id = $id;
		$this->value = $value;
		$this->moduleId = $moduleId;
		
		$this->dataFrom = $dataFrom;
		$this->dataTo = $dataTo;
		
		$this->classifications = array();
		$this->measurements = array();
		
		// Collect all classifications and measurements	
		$this->reportData = array();
		
	}
	
	public function getUID() {
		return $this->uid;
	}

	public function getId() {
		return $this->uid;
	}
	
	public function getValue() {
		return $this->value;
	}
	
	public function getModuleId() {
		return $this->moduleId;
	}
	
	public function getDataFrom() {
		return $this->dataFrom;
	}
	
	public function getDataTo() {
		return $this->dataTo;
	}

	public function getMeasurements() {
		if (empty($this->measurements)) {
			$this->measurements = DatabaseManager::getMeasurements($this);
		}
		return $this->measurements;
	}
	
	public function &getClassifications() {
		if (empty($this->classifications)) {
			$this->classifications = DatabaseManager::getClassifications($this);
		}
		return $this->classifications;
	}

	/**
	 * @return Classification
	 */
	public function getClassification($uid) {
		if (empty($this->classifications)) {
			$this->classifications = DatabaseManager::getClassifications($this);
		}
		
		foreach ($this->classifications as $classification) {
			if ($classification->getUID() == $uid) {
				return $classification;
			}
		}
		
		return null;
	}

	
	/**
	 * @return Classification
	 */
	public function getClassificationByInternalId($id) {
		if (empty($this->classifications)) {
			$this->classifications = DatabaseManager::getClassifications($this);
		}
		
		foreach ($this->classifications as $classification) {
			if ($classification->getId() == $id) {
				return $classification;
			}
		}
		return null;
	}
	
	
	/* used with sort */
	public function sort($a, $b) {
			
		//if (!$a instanceof Template || !$b instanceof Template) return 0;
	
		if (strtoupper($a->reportData["title"]) == strtoupper($b->reportData["title"])) return 0;
		return (strtoupper($a->reportData["title"]) > strtoupper($b->reportData["title"])) ? 1 : -1;
		
	}
	
	
	public function isSelectionValid() {
		$classifications = $this->getClassifications();
		foreach ($classifications as $classification) {
			if (!$classification->isSelectionValid()) {
				return false;
			}
		}		
		return true;
	}
	
	
	public function getSelectedClassificationValues() {
		$values = array();
	}
	
	public function getNumberOfSelectedClassifications() {
		$count = 0;
		foreach ($this->classifications as $classification) {
			if ($classification->isSelected()) $count++;
		}
		return $count;
	}
	
	public function getNumberOfSelectedMeasurements() {
		$count = 0;
		foreach ($this->measurements as $measurement) {
			if ($measurement->isSelected()) $count++;
		}
		return $count;		
	}
	
	public function isUserInputEnabled() {
		foreach ($this->classifications as $classification) {
			if ($classification->isAllowInput()) return true;
		}
		return false;
	}
	
	public function refreshAccessRights() {
		foreach ($this->classifications as $classification) {
			$useAnonymous = $_SESSION["USER"]->isClassificationAnonymous($this->getId(), $classification->getId(), $this->reportData["folder"]);
			$classification->setUseAnonymous($useAnonymous);
		}
	}
	

	public function __toString() {
		return serialize($this);
	}
}


class Template extends DataType {
	
	private $uid;
	
	private $acctype;	
	private $accvars;
	private $folder;
	private $title;
	
	private $pres;

	// used for other variables
	private $variables;
	
	private $sections;
	private $classification;
	
	// User that has loaded the template
	private $user;
	
	function __construct() {
		$this->uid = time() . rand(1000, 9999);
		$this->sections = array();
	}
	
	function load() {
		if (!empty($section)) return;
		foreach ($this->sections as $section) {
			$section->load();
		}
	}

	function getUid() {
		return $this->uid;
	}
	
	public function setAcctype($acctype) {
		$this->acctype = $acctype;
		$dataLoader = new DataLoader();
		$this->classification = $dataLoader->getClassification($acctype - 1);
	}
	
	
	public function setUser(User $user) {
		$this->user = $user;
	}
	
	public function getUser() {
		return $this->user;
	}
	
	public function setAccvars($accvars) {
		$this->accvars = $accvars;
	}

	public function getAccvars() {
		return $this->accvars;
	}
	
	public function getAcctype() {
		return $this->acctype + 1;
	}
	
	public function setClassvars($classvars) {
		$classvars = explode(":", $classvars);
		foreach ($classvars as $inertnalId => $classvar) {
			if ($classvar != "x") {
				
				// Remove first and last char
				$classvar = substr($classvar, 1, -1);
				
				$section = new TemplateSection($this, $this->acctype);
				$section->setClassificationInternalId($inertnalId);
				
				// Check if the selection is controlled
				if ($classvar != "") {
					
					// check if the user can have any influence on the selection
					$controller = substr($classvar, 0, 1);
					$values = strlen($classvar) > 1 ? explode(",", substr($classvar, 2)) : array();
										
					if ($controller == "*") {
						$section->setUserInput(true);
						$section->addValues($values);
					}
					else if ($controller == "+") {
						$section->setUserInput(true);
						$section->setMin(1);
						$section->addValues($values);
					}
					else if ($controller == ".") {
						$section->setUserInput(true);
						$section->setMin(1);
						$section->setMax(1);
						$section->addValues($values);
					}
					else {
						$section->setUserInput(false);
						$values = explode(",", $classvar);
						$section->addValues($values);
					}
					
					
				}
				
				$this->sections[] = $section;
			}
		}
	}
	
	public function getSections() {
		return $this->sections;
	}
	
	public function getSection($classificationId) {
		foreach ($this->sections as $section) {
			if ($section->getClassificationInternalId() == $classificationId) {
				return $section;
			}
		}
	}
	
	public function checkUserInput() {
		
		$wrongInput = array();		
		$sections = $this->sections;
		
		/*@var $section TemplateSection */
		foreach ($sections as $section) {
			if ($section->isUserInput()) {				
				if (!$section->isSelectionValid()) {
					$wrongInput[] = $section->getClassificationInternalId();	
				}
			}
		}
		return $wrongInput;
	}
	
	
	public function setFolder($folder) {
		$this->folder = $folder;
	}
	
	public function setTitle($title) {
		$this->title = $title;
	}
	
	public function getTitle() {
		return $this->title;
	}
	
	
	public function getFolder() {
		return $this->folder;
	}
	
	public function setPres($pres) {
		$this->pres = $pres;
	}
	
	public function getPres() {
		return $this->pres;
	}

	public function set($key, $value) {
		$this->variables[$key] = $value;
	}
	
	public function get($key) {
		return !empty($this->variables[$key]) ? $this->variables[$key] : null;
	}
	
	public function getVariables() {
		return $this->variables;
	}

	public function isUserInput() {
		$needUserInput = false;
		foreach ($this->sections as $section) {
			if ($section->isUserInput()) {
				$needUserInput = true;
			}
		}
		return $needUserInput;
	}
		
	
	/* used with sort */
	function sort($a, $b) {
		
		if (!$a instanceof Template || !$b instanceof Template) return 0;
	
		if (strtoupper($a->getTitle()) == strtoupper($b->getTitle())) return 0;
		return (strtoupper($a->getTitle()) > strtoupper($b->getTitle())) ? 1 : -1;
		
	}
	
	/**
	 * Needed by PHP 5.2.0
	 */
	public function __toString() {
		return serialize($this);
	}
	
	
}

?>