<?php

/*
 * Reads a simple filter map file, and let the user
 * exclude a certaion feature for a particular product name
 */
class Filter {

	/**
	 * Comments starts with #
	 *
	 * Key word is: "EXCLUDE: " connected with the combind name e.g. "EXCLUDE: SMT"
	 * Every line after this keyword, contains of a feature name that will excluded
	 * when showed in the WebGUI.
	 *
	 * Product names can use wildcars '?' to match any character or
	 * '*' to match any substring (0 or more characters).
	 *
	 */


	private $file;
	private $map;


	public function Filter($filename = null) {

		if (is_null($filename)) {
			throw new Exception("We need a exclud-mapping file..");
		}

		/*if (is_file($filname)) {
			throw new Exception("Could not find map file {$filname}");
			}*/

		$this->map = array();
		$this->file = $filename;

		$this->load();
	}

	private function load() {

		if (!file_exists($this->file)) return;

		// Parse the mapping file, to an easier format for looking up data..
		$parser = new Parser(file_get_contents($this->file),
		ParserSettings::create(ParserSettings::COMMENT_CHAR, '#')
		->set(ParserSettings::SKIP_COMMENTS, true)
		->set(ParserSettings::SKIP_BLANK_LINES, true)
		);

		$product = null;

		// For each line we have...
		while ($parser->hasMore()) {

			// Get next line
			$line = $parser->next();

			// Check if got the keyword
			if (strtolower(substr(trim($line->toString()), 0, 8)) == 'exclude:') {
				$map = $line->next(ParserSettings::create(ParserSettings::SPLIT, ParserSettings::SPLIT_TYPE_SEMI_COLON));
				$product = trim($map->get(1)->toString());
				$this->map[$product] = array();
				continue;
			}

			// if we dont have a product name, we continue..
			if (is_null($product)) continue;


			// add product name to combined product name
			$this->map[$product][] = trim($line->toString());
		}
	}

	public function isStringExcluded($productName,$featureVersion) {
		$prod = $this->map[$productName];
		if($prod){
			foreach ($prod as $value) {
				$value = trim(strtolower($value));

				if (eregi($value, $featureVersion)) {
					return "true" . $value . $featureVersion;
				}
			}
		}
		return "false" . $value . $featureVersion;
	}

	public function isExcluded($productName,$featureVersion){
		if(count($this->map) <= 0) return null;
		$prod = $this->map[$productName];
		if($prod){
		foreach ($prod as $value) {
			$value = trim(strtolower($value));

			if (eregi($value, $featureVersion)) {
				return true;
			}
		}
		}
		return false;
	}
}

?>