<?php

class Group {

	// Error
	const ERROR_GROUPEXISTS = 10;
	const ERROR_NONAME = 20;
	
	private $id;
	private $name;
	private $description;
	
	private $groups;
	private $datatypes;
	private $products;
	private $features;
	private $folders;
	private $anonymousClassifications;
	private $contentLimits;
	
	public function __construct($id = null, $name = null, $description = null) {
		$this->id = $id;
		$this->name = $name;
		$this->description = $description;
	}
	
	
	public function getId() {
		return $this->id;
	}
	
	public function getName() {
		return $this->name;
	}
		
	public function getDescription() {
		return $this->description;
	}
	
	public function setId($id) {
		$this->id = $id;
	}
	
	public function setName($name) {
		$this->name = $name;
	}
	
	public function setDescription($description) {
		$this->description = $description;
	}
	
	
	public function getGroups() {
		if (empty($this->groups) && $this->id != "") {
			$this->groups = AccessControlManager::getGroupMembers($this->id);
		}
		return $this->groups;
	}

	public function isMemberOf($group) {
		if ($this->id == "") return array();
		
		if (!is_array($this->groups)) {
			$this->groups = AccessControlManager::getGroupMembers($this->id);
		}
		if (in_array($group, $this->groups)) {
			return true;
		}
		
		return false;
	}
	
	public function getDataTypeMembers() {
		if ($this->id == "") return array();
		
		if (!is_array($this->datatypes)) {
			$this->datatypes = AccessControlManager::getGroupDataTypes($this->id, false);
		}
		
		return $this->datatypes;
	}
	
	public function isMemberOfDataType($datatypeId) {		
		$dataTypeMember = $this->getDataTypeMembers();
		if (in_array($datatypeId, $dataTypeMember)) {
			return true;
		}
		return false;
	}	
	
	public function isClassificationAnonymous($dataTypeId, $classificationId) {
		if ($this->id == "") return null;
		
		if (!is_array($this->anonymousClassifications)) {
			$this->anonymousClassifications = AccessControlManager::getAnonymousClassificationInGroup($this);
		}
		Logger::Debug("GROUP", "CHECK FOR ANON : $dataTypeId");
		if (!isset($this->anonymousClassifications[$dataTypeId])) return false;
		Logger::Debug("GROUP", "CHECK FOR : $classificationId");
		if (in_array($classificationId, $this->anonymousClassifications[$dataTypeId])) {
			Logger::Debug("GROUP", "ANON!");
			return true;
		}
		return false;
	}
	
	
	public function getAnonymousClassifications() {
		if ($this->id == "") return array();
		
		if (!is_array($this->anonymousClassifications)) {
			$this->anonymousClassifications = AccessControlManager::getAnonymousClassificationInGroup($this);
		}
		return $this->anonymousClassifications;
	}
	
	public function getProducts() {
		if ($this->id == "") return array();
		
		if (!is_array($this->products)) {
			$this->products = AccessControlManager::getGroupProducts($this->id);
		}
		return $this->products;
	}

	public function isMemberOfProduct($product) {
		if ($this->id == "") return array();
		
		if (!is_array($this->products)) {
			$this->products = AccessControlManager::getGroupProducts($this->id, false);
		}
		if (in_array($product, $this->products)) {
			return true;
		}
		return false;
	}

	public function isMemberOfFeature($feature) {
		if ($this->id == "") return array();
		
		if (!is_array($this->features)) {
			$this->features = AccessControlManager::getGroupFeatures($this->id, false);
		}
		
		if (in_array($feature, $this->features)) {
			return true;
		}
		
		return false;
	}
	
	public function getFolders() {
		if ($this->id == "") return array();
		
		if (!is_array($this->folders)) {
			$this->folders = AccessControlManager::getGroupFolder($this->id);
		}
		return $this->folders;
	}

	public function isMemberOfFolder($folder) {
		if ($this->id == "") return array();
		
		if (!is_array($this->folders)) {
			$this->folders = AccessControlManager::getGroupFolder($this->id);
		}
		if (in_array($folder, $this->folders)) {
			return true;
		}
		return false;
	}

	public function store() {
		return AccessControlManager::updateGroup($this);
	}
	
	public function getContentLimit($classificationId) {
		if ($this->id == "") return array();
		
		if (!is_array($this->contentLimits)) {
			$this->contentLimits = AccessControlManager::getContentLimit($this);
		}
		
		foreach ($this->contentLimits as $contentLimit) {
			if ($contentLimit->getClassificationInternalId() == $classificationId) {
				return $contentLimit;
			}
		}
		return false;
	}

}

?>