<?php

class MapFilter {
	
	/**
	 * Comments starts with #
	 * 
	 * Key word is: "MAP: " connected with the combind name e.g. "MAP: Geoframe"
	 * Every line after this keyword, contains of a product name that will be mapped
	 * to this combind name.
	 * 
	 * Product names can use wildcars '?' to match any character or 
	 * '*' to match any substring (0 or more characters).
	 * 
	 */

	
	private $file;
	public $filters;
	
	public function MapFilter($filname = null) {
		
		if (is_null($filname)) {
			throw new Exception("We need a map filter file..");
		}
		
		//$this->map = array();
		$this->file = $filname;
		
		$this->load();
	}
	
	private function load() {
		
		if (!file_exists($this->file)) return;
		
		// Parse the map filter file, to an easier format for looking up data..
		$parser = new Parser(file_get_contents($this->file),
			ParserSettings::create(ParserSettings::COMMENT_CHAR, '#')
			//->set(ParserSettings::COMMENT_CHAR, '}')
			->set(ParserSettings::SKIP_COMMENTS, true)
			->set(ParserSettings::SKIP_BLANK_LINES, true)
		);
		
		$product = null;
		$count=0;
		// For each line we have...
		while ($parser->hasMore()) {
			
			// Get next line
			$line = $parser->next();
			
			//skip closing braces
			if(trim($line->toString() )== '}')continue;
			
			if (trim($line->toString() )== 'filter {') {
				$count++;
				$this->filters[$count] = array();
				
				continue;
			}

			// add map filter rule
			$filters = $line->next(ParserSettings::create(ParserSettings::SPLIT, ParserSettings::SPLIT_TYPE_EQUALS));
			//$product = trim($filters->get(1)->toString());
			//$name=MapFilter::regexCorrection(trim($filters->get(0)->toString()));
			$rCheck=MapFilter::regexCorrection(trim($filters->get(1)->toString()));
			$this->filters[$count][trim($filters->get(0)->toString())] = $rCheck;
		}
		
	}
	
	//Replaces regex intended for PERL into ones usable by PHP(preg_match) 
	private function regexCorrection($rCheck){
		switch($rCheck){
			case '*':
				return '.*';
				break;
			default:
				return $rCheck;
		}
	}
}

?>