<?php

// 
define("REPORTLOADER_CACHE", "reportLoaderCache");
define("REPORTLOADER_CACHE_MD5", "reportLoaderCacheMD5");
define("REPORTLOADER_SECTION_FOLDERS", "SERVER_FOLDERS");
define("REPORTLOADER_SECTION_REPORTS", "SERVER_REPORTS");

class ReportLoader {
	
	private $data;
	private $md5;
	private $folders;
	private $reports;
	
	public function __construct() {
		$this->load();
	}
	
	function __destruct() {
		$_SESSION[REPORTLOADER_CACHE] = array();
		$_SESSION[REPORTLOADER_CACHE]["FOLDERS"] = $this->folders;
		$_SESSION[REPORTLOADER_CACHE]["REPORTS"] = $this->reports;
		$_SESSION[REPORTLOADER_CACHE_MD5] = $this->md5;
	}
	
	
	private function load() {		

		$this->md5 = null;
		
		if (isset($_SESSION[REPORTLOADER_CACHE]) && isset($_SESSION[REPORTLOADER_CACHE_MD5])) {
			$this->md5		= $_SESSION[REPORTLOADER_CACHE_MD5];
			$this->folders 	= $_SESSION[REPORTLOADER_CACHE]["FOLDERS"];
			$this->reports 	= $_SESSION[REPORTLOADER_CACHE]["REPORTS"];
		}

		// Get data
		$this->data = file(ServerConfig::getServerUrl() . "cron_admin.pl?command=list");
		$md5 = md5(implode("", $this->data));
		
		// Check if we need to parse the data
		if (is_null($md5) || $this->md5 != $md5 || empty($this->reports)) {
			$this->md5 = $md5;
			$this->folders 	= array();
			$this->reports 	= array();
			$this->parse();
		}
		
	}
	
	private function parse() {
		
		$mode = null;
		$reports = array();
		$report = null;
					
		foreach ($this->data as $line) {
			$line = trim($line);
			
			// Setting mode
			if ($line == REPORTLOADER_SECTION_FOLDERS) {
				$mode = REPORTLOADER_SECTION_FOLDERS;
				continue;
			} else 
			if ($line == REPORTLOADER_SECTION_REPORTS) {
				$mode = REPORTLOADER_SECTION_REPORTS;
				continue;
			}
			
			// If blank line, we skipp paresing
			if (empty($line)) continue;
			
			// Parse data
			switch ($mode) {
				
				// Folder options
				case REPORTLOADER_SECTION_FOLDERS:
					$this->folders[] = new Folder($line);
					break;

				case REPORTLOADER_SECTION_REPORTS:
					list($action, $data) = explode(":", $line, 2);
					
					switch ($action) {
						
						case "ID":
							unset($report);
							$report = new Report($data);
							$reports[] =& $report;
							break;
						
						case "TYPE":
							$report->setType($data);
							break;
							
						case "PERTITLE":
							$report->setTitle($data);
							break;
							
						case "TITLE":
							if ($report->getType() == "R") {
								$reportInstance = new Report($report->getId());
								$report->addInstance($reportInstance);								
								$reportInstance->setTitle($data);
								$reportInstance->setType('R');
								break;
							}
							$report->setTitle($data);
							break;
							
						case "FREQ":
							$report->setFreq($data);
							break;

						case "ACTIVE":
							$report->setActive($data);
							break;

						case "TOFOLDER":
							$report->setFolder($data);
							break;
							
						case "FOLDER":
							if ($report->getType() == "R") {
								$reportInstance->setFolder($data);
								break;
							}
							$report->setFolder($data);
							break;
							
						case "PRES":
							$report->setPres($data);
							break;
							
						case "PERIOD":
							if ($report->getType() == "R") {
								$reportInstance->setPeriod($data);
								break;
							}
							$report->setPeriod($data);
							break;
							
						case "INS":
							if ($report->getType() == "R") {
								$reportInstance->setIns($data);
								break;
							}
							$report->setIns($data);
							break;
							
							
					}
					
					break;
			}
			
		}
		
		// Sorting reports
		usort($reports, "Report_sort");
		
		$this->reports = $reports;
	}
	
	
	public function getFolders() {
		return $this->folders;
	}
	
	public function getReportByTemplateId($id, $parentId = null) {
		/* @var $report Report */
		foreach ($this->reports as $report) {
			if ($report->getTemplateId() == $id) {
				return $report;
			}
			
			// If the report is an Period Definition, we need to check its instances
			if ($parentId != null && $report->getId() == $parentId && $report->hasInstaces()) {
				$subReports = $report->getInstances();

				foreach ($subReports as $subReport) {										
					if ($subReport->getTemplateId() == $id) {						
						return $subReport;
					}
				}
			}
		}
		Logger::Debug("ReportLoader::getReportByTemplateId", "Could not found template with id $id");
		return false;
	}
	
	public function getReport($id) {
		foreach ($this->reports as $report) {
			if ($report->getId() == $id) {
				return $report;
			}
		}
		return false;		
	}
	
	public function removeReportByTemplateId($id, $parentId) {
		$report = $this->getReportByTemplateId($id, $parentId);
		if ($report) {
	//		die(ServerConfig::getServerUrl() . "cron_admin.pl?i=" . urlencode($report->isSeen() ? "+" : "-") . $id . "&t=" . $report->getId() . "&folder=" . $report->getFolder() . "&command=delinstance");
			$result = file_get_contents(ServerConfig::getServerUrl() . "cron_admin.pl?i=" . urlencode($report->isSeen() ? "+" : "-") . $id . "&t=" . $report->getId() . "&folder=" . $report->getFolder() . "&command=delinstance");
			if (strtoupper(trim($result)) == "OK") {
				return true;
			}
		}
		return false;
	}
	
	public function moveReportByTemplateId($id, $parentId, $newFolder) {
		$report = $this->getReportByTemplateId($id, $parentId);
		if ($report) {
			$result = file_get_contents(ServerConfig::getServerUrl() . "cron_admin.pl?movefrom=" . urlencode($report->getFolder()) . "&i=" . urlencode($report->isSeen() ? "+" : "-") . $id . "&t=" . $report->getId() . "&moveto=" . urlencode($newFolder) . "&command=moveinstance");
			if (strtoupper(trim($result)) == "OK") {
				return true;
			}
		}
		return false;		
	}
	
	public function renameReportByTemplateId($id, $newName) {
		$report = $this->getReportByTemplateId($id);
		if ($report) {
			$result = file_get_contents(ServerConfig::getServerUrl() . "cron_admin.pl?folder=" . urlencode($report->getFolder()) . "&i=" . urlencode($report->isSeen() ? "+" : "-") . $id . "&t=" . $report->getId() . "&newname=" . urlencode($newName) . "&command=renameinstance");
			if (strtoupper(trim($result)) == "OK") {
				return true;
			}
		}
		return false;		
	}

	public function activatePeriodic($id) {
		$report = $this->getReport($id);
		if ($report) {
			$result = file_get_contents(ServerConfig::getServerUrl() . "cron_admin.pl?folder=" . urlencode($report->getFolder()) . "d&id=" . $id . "&command=activate");
			if (strtoupper(trim($result)) == "OK") {
				return true;
			}
		}
		return false;		
	}

	public function deactivatePeriodic($id) {
		$report = $this->getReport($id);
		if ($report) {
			$result = file_get_contents(ServerConfig::getServerUrl() . "cron_admin.pl?folder=" . urlencode($report->getFolder()) . "d&id=" . $id . "&command=deactivate");
			if (strtoupper(trim($result)) == $id) {
				return true;
			}
		}
		return false;		
	}
	
	public function deletePeriodic($id) {
		$report = $this->getReport($id);
		
		if ($report) {
			$result = file_get_contents(ServerConfig::getServerUrl() . "cron_admin.pl?folder=" . urlencode($report->getFolder()) . "&id=" . $id . "&command=delete");
			if (strtoupper(trim($result)) == "OK") {
				return true;
			}
		}
		return false;		
	}
	
	
	private function sort($reports, $sort = null, $asc = true) {

		// No sorting, return all reports as they are,,
		if (is_null($sort)) {
			return $this->reports;
		}

		// Do some sorting
		if ($sort == "title") {
			usort($reports, array(new Report(), "sortTitle"));	
		} else if ($sort == "folder") {
			usort($reports, array(new Report(), "sortFolder"));
		} else {
			usort($reports, array(new Report(), "sortDate"));
		}
		
		// Asc or desc?
		if ($asc) return array_reverse($reports);
		return $reports;				
	}

	
	/**
	 * Was the report array based on user folder access
	 *
	 * @param Array $reports Array with Reports
	 * @return Array A washed Array with Reports
	 */
	private function accessWash($reports) {

		// Check access rights
		if (!$_SESSION["USER"]->isAdmin()) {
			
			// folders the user has access to
			$folders = $_SESSION["USER"]->getFolders();
			
			$userReports = array();
			
			foreach ($reports as $report) {
				if (in_array(new Folder($report->getFolder()), $folders)) {
					$userReports[] = $report;
				}
			}
			
			// set it back to reports
			$reports = $userReports;
		}

		return $reports;
	}
	
	public function getPeriodicJobs($sort = null, $asc = true) {
		$reports = array();
		
		foreach ($this->reports as $report) {	
			if ($report->getType() == "R") {
				$reports[] = $report;
			}			
		}
		
		// Access check
		$reports = $this->accessWash($reports);
		
		// Return it sorted..
		return $this->sort($reports, $sort, $asc);		
	}
	
	public function getReports($folder = null, $sort = null, $asc = true) {
		$reports = array();
		
		foreach ($this->reports as $report) {	
			if ($report->getType() != "R" && ((is_null($folder) || $folder == "*") || $report->getFolder() == $folder)) {
				$reports[] = $report;
			}			
			else if ($report->getType() == "R") {
				$subReports = $report->getInstances();
				foreach ($subReports as $subReport) {
					if ((is_null($folder) || $folder == "*") || $subReport->getFolder() == $folder) {
						$reports[] = $subReport;
					}		
				}					
			}			
		}
		
		// Access check
		$reports = $this->accessWash($reports);
						
		// Return it sorted..
		return $this->sort($reports, $sort, $asc);
	}

	
	/**
	 * @deprecated 
	 */
	public function setReload($reload) { }

}

function Report_sort($a, $b) {
        return strcasecmp($a->folder, $b->folder);
}

?>