<?php

/**
 * ServerConfig
 * 
 * This class holds all the variable information/configuration for the program.
 *
 * @author Glenn O Larsen <glenn.larsen@openit.com>
 * @copyright (C) Copyright 1997-2006 by Open iT Inc, All rights reserved.
 * 
 */
class ServerConfig {
	
	const INI_SERVER = "server";
	const INI_CLIENT = "client";
	const INI_PORTNUMBER = "port";
	const INI_DATABASE = "database";
	const INI_LOGFILE = "logfile";
	const INI_LOGLEVEL = "loglevel";
	const INI_TITLE = "title";
	const INI_CHARSET = "charset";
	const INI_DBVERSION = "dbversion";
	const INI_LANGUAGE = "language";
	const INI_WEEKFORMAT = "weekformat";	
	const INI_REUSEREPORTWINDOW = "reuse_report_window";
	const INI_PASSWORDBACKEND = "passwordbackend";	
	const INI_LICENSE_STATUS = "license_status";
	const INI_TOPBANNER = "topbanner";
	const INI_BOTBANNER = "botbanner";
		
	static private $configPath;
	static private $databasePath;
	static private $languagePath;
	static private $database;
	static private $serverUrl;
	static private $clientUrl;
	
	private $iniValues;
	
	// NB: Default config.ini files is created in SystemChecker class
	
	static private $instance;

	
	public function __construct() {		
		
		/* Default configuration */
		$this->iniValues = array(
			self::INI_SERVER 			=> "",
			self::INI_CLIENT 			=> "",
			self::INI_PORTNUMBER		=> "",
			self::INI_DATABASE 			=> "",
			self::INI_TITLE 			=> "",
			self::INI_LOGFILE 			=> "",
			self::INI_TOPBANNER 		=> "",
			self::INI_BOTBANNER 		=> "",
			self::INI_LOGLEVEL 			=> 0,
			self::INI_CHARSET 			=> "UTF-8",
			self::INI_LANGUAGE 			=> "en",		
			self::INI_WEEKFORMAT		=> "week_number",
			self::INI_REUSEREPORTWINDOW	=> 0,
			self::INI_PASSWORDBACKEND	=> AccessControlManager::PASSWORDBACKEND_LOCAL,
			self::INI_LICENSE_STATUS	=> 1,
			self::INI_DBVERSION			=> "1"
		);
				
		// Make sure we have a document root
		if (is_null($_SERVER["DOCUMENT_ROOT"])) {
			$_SERVER["DOCUMENT_ROOT"] = ".";

		}	

		// Check if we have INI settings in session, if not, we put it to session
		if (!isset($_SESSION["INI"])) {
			
			// Check if we find the genral.ini file, if not go to error page
			if (!file_exists($_SERVER["DOCUMENT_ROOT"] . "/../config/general.ini")) {
				die("Can't locate the Open iT Server Web GUI configuration file.<br />Please make sure that this file exist and has the right permission set.");
				return;
			}
			
			// Get data from the INI file
			$ini = parse_ini_file($_SERVER["DOCUMENT_ROOT"] . "/../config/general.ini");
			
			// Pre-peare the SESSION to have a array with the configuration settings
			$_SESSION["INI"] = array();
			
			// Read thru all the keys, and set the given value (or default value if no value is given)
			foreach ($this->iniValues as $iniKey => $defaultValue) {
				$_SESSION["INI"][$iniKey] = isset($ini[$iniKey]) ? $ini[$iniKey] : $defaultValue;
			}
			
		}
		
		// Get values
		self::$serverUrl = $_SESSION["INI"][self::INI_SERVER];
		self::$clientUrl = ($_SESSION["INI"][self::INI_CLIENT] == "") ? self::$serverUrl : $_SESSION["INI"][self::INI_CLIENT];
		
		//check for the path of httpd.conf
		if (! strstr(PHP_OS, "WIN")){
			$br = new BackendReader("/var/opt/openit/etc/httpd/httpd.conf");
		}else{
			$br = new BackendReader($_SERVER["DOCUMENT_ROOT"] . "/../../Configuration/httpd/httpd.conf");
		}
		
		// If the config.ini file has <blank> as an entry - we use hostname as the URL for server location
		if (self::$serverUrl == "") {

			$port = isset($_SERVER["SERVER_PORT"]) && $_SERVER["SERVER_PORT"] != 80 ? ":" . $_SERVER["SERVER_PORT"] : "";
			$ssl = file_exists('/var/opt/openit/etc/httpd/use_ssl');
			// If we use SSL, we use the "hidden" http port for Open iT Web GUI (since we cant use https for backend stuff)
			if ($ssl || isset($_SERVER["HTTPS"]) && $_SERVER["HTTPS"] == "on") {
					self::$serverUrl = "http://127.0.0.1:" . $br->getPort() ."/OpeniT/";
					self::$clientUrl = "https://";
				} else {
					self::$serverUrl = "http://127.0.0.1" . $port . "/OpeniT/";
					self::$clientUrl = "http://";
				}

			self::$clientUrl .= (isset($_SERVER["HTTP_HOST"]) ? $_SERVER["HTTP_HOST"] : "127.0.0.1" . $port) . "/OpeniT/";
			
		}
		
		self::$databasePath = $_SERVER["DOCUMENT_ROOT"] . "/../db/data/";
		self::$configPath = $_SERVER["DOCUMENT_ROOT"] . "/../config/";
		self::$languagePath = $_SERVER["DOCUMENT_ROOT"] . "/../smarty/languages/";	
	}
	
	static private function checkInstance($force = false) {
		if (empty(self::$instance) || !isset($_SESSION["INI"]) || $force) {
			self::$instance = new ServerConfig();
		}	
	}

	
	static private function updateConfigFile($iniKey, $newValue) {
		$_SESSION["INI"][$iniKey] = $newValue;
		$ini = parse_ini_file($_SERVER["DOCUMENT_ROOT"] . "/../config/general.ini");
		$fh = fopen($_SERVER["DOCUMENT_ROOT"] . "/../config/general.ini", "w");
		$valueIsSet = false;
		foreach ($ini as $key => $value) {
			if ($key == $iniKey) {
				$valueIsSet = true;
				fputs($fh, $key . '=' . '"'.$newValue.'"' . "\n");
			} else {
				fputs($fh, $key . '=' . '"'.$value.'"' . "\n");
			}
		}
		
		// In case its not here before, we sett it at the end..
		if (!$valueIsSet) {
			fputs($fh, $iniKey . '=' . '"'.$newValue.'"' . "\n");
		}
		
		fclose($fh);
	}
	
	static function getDatabase() {
		self::checkInstance();
		return self::$databasePath . $_SESSION["INI"][self::INI_DATABASE];
	}
	
	static function setDatabase($databaseFilename) {
		self::updateConfigFile(self::INI_DATABASE , $databaseFilename);
	}
	
	static function getDatabasePath() {
		self::checkInstance();
		return self::$databasePath;
	}
	
	static function getLanguagePath() {
		self::checkInstance();
		return self::$languagePath;
	}
	
	static function getConfigPath() {
		self::checkInstance();
		return self::$configPath;
	}
	
	static function getServerUrl() {
		self::checkInstance();
		return self::$serverUrl;		
	}

	static function getClientUrl() {
		self::checkInstance();
		return self::$clientUrl;
	}

	
	static function getServer() {
		self::checkInstance();
		return $_SESSION["INI"][self::INI_SERVER];
	}
	
	static function setServer($server) {
		self::updateConfigFile(self::INI_SERVER, $server);
	}

	static function getClient() {
		self::checkInstance();
		return $_SESSION["INI"][self::INI_CLIENT];
	}
	
	static function setClient($client) {
		self::updateConfigFile(self::INI_CLIENT, $client);
	}
	
	static function getLogFile() {
		self::checkInstance();
		if (isset($_SESSION["INI"][self::INI_LOGFILE]) && trim($_SESSION["INI"][self::INI_LOGFILE]) != "") {
			if (strstr($_SESSION["INI"][self::INI_LOGFILE], "/") || strstr($_SESSION["INI"][self::INI_LOGFILE], "\\")) {
				return stripslashes ($_SESSION["INI"][self::INI_LOGFILE]);
			} else {
				return stripslashes (rtrim($_SERVER["DOCUMENT_ROOT"], '/') . "/../logs/" . $_SESSION["INI"][self::INI_LOGFILE]);
			}
		}
		return "";
	}
	
	static function setLogFile($logfile) {
		self::updateConfigFile(self::INI_LOGFILE, $logfile);
		@Logger::createLogFile();
	}
	
	
	static function getLogLevel() {
		self::checkInstance();
		return $_SESSION["INI"][self::INI_LOGLEVEL];
	}
	
	static function setLogLevel($loglevel) {
		if (is_numeric($loglevel) && $loglevel >= 0 && $loglevel <= 4) {
			self::updateConfigFile(self::INI_LOGLEVEL , $loglevel);
		}
	}	
	
	static function getTitle() {
		self::checkInstance();
		return $_SESSION["INI"][self::INI_TITLE];
	}

	static function setTitle($title) {
		self::updateConfigFile(self::INI_TITLE, $title);
	}

	
	/**
	 * Return available languages for the given charset
	 *
	 * @return Mixed
	 */
	static function getAvailableCharset() {
		$files = glob(self::$languagePath . "/*");
		$charsets = array();
		foreach ($files as $file) {
			if (is_dir($file)) {
				$charsets[] = strtoupper(basename($file));
			}
		}
		return $charsets;
	}

	static function getCharset() {
		self::checkInstance();
		return $_SESSION["INI"][self::INI_CHARSET];
	}
	
	static function setCharset($charset) {
		self::updateConfigFile(self::INI_CHARSET, strtoupper($charset));
	}
	
	/**
	 * Return available languages for the given charset
	 *
	 * @return Mixed
	 */
	static function getAvailableLanguages() {
		$files = glob(self::$languagePath . "/" . strtolower(self::getCharset()) . "/*.ini");
		$languages = array();
		foreach ($files as $file) {
			$ini = parse_ini_file($file);
			if (isset($ini["language"])) {
				$shortname = substr(basename($file), 0, strpos(basename($file), "."));	//	path/path/no.ini => no
				$languages[$shortname] = $ini["language"];
			}
		}
		return $languages;
	}
	
	static function getDbVersion($force = false) {
		self::checkInstance($force);
		return $_SESSION["INI"][self::INI_DBVERSION];		
	}
	
	static function setDbVersion($version) {
		self::updateConfigFile(self::INI_DBVERSION, $version);
	}
		
	static function getDefaultLanguage() {
		self::checkInstance();
		return $_SESSION["INI"][self::INI_LANGUAGE];
	}
	
	static function setDetfaultLanguage($language) {
		self::updateConfigFile(self::INI_LANGUAGE, $language);
	}		
	
	static function getWeekFormat() {
		self::checkInstance();
		return $_SESSION["INI"][self::INI_WEEKFORMAT];
	}
	
	static function setWeekFormat($weekFormat) {
		self::updateConfigFile(self::INI_WEEKFORMAT, $weekFormat);
	}
		
	static function getReuseReportWindow() {
		self::checkInstance();
		return $_SESSION["INI"][self::INI_REUSEREPORTWINDOW];
	}
	
	static function setReuseReportWindow($boolean) {
		self::updateConfigFile(self::INI_REUSEREPORTWINDOW, $boolean);
	}
	
	
	static function getPasswordBackend() {
		self::checkInstance();
		return $_SESSION["INI"][self::INI_PASSWORDBACKEND];
	}
	
	static function setPasswordBackend($backend) {
		self::updateConfigFile(self::INI_PASSWORDBACKEND, $backend);
	}
	

	// License Status
	static function getLicenseStatus() {
		self::checkInstance();
        return 1; // Should not be used - use registry insted
		return $_SESSION["INI"][self::INI_LICENSE_STATUS];
	}
	
	static function setLicenseStatus($boolean) {
		self::updateConfigFile(self::INI_LICENSE_STATUS, $boolean);
	}
	
	// Port Number
	static function getPortNumber($force = false) {
		self::checkInstance($force);
		return $_SESSION["INI"][self::INI_PORTNUMBER];		
	}
	
	static function setPortNumber($port) {
		self::updateConfigFile(self::INI_PORTNUMBER, $port);
	}
	
	// Top Banner
	static function getTopBanner() {
		self::checkInstance();
		return $_SESSION["INI"][self::INI_TOPBANNER];		
	}
	
	static function setTopBanner($topban) {
		self::updateConfigFile(self::INI_TOPBANNER, $topban);
	}
	
	// Bottom Banner
	static function getBottomBanner() {
		self::checkInstance();
		return $_SESSION["INI"][self::INI_BOTBANNER];		
	}
	
	static function setBottomBanner($botban) {
		self::updateConfigFile(self::INI_BOTBANNER, $botban);
	}
	
}


?>