<?php

/**
 * 
 * This class should verify that the system is up-to-date, and should work correct.
 * 
 * This class should also, include all things done by "updateing" a version.
 * 
 */


class SystemChecker {

	const VERSION = 3;
	
	const ERROR_INI_FILE_CANT_OPEN = 1;
	const ERROR_INI_FILE_WRITE_ACCESS = 2;
	
	public static function run($force = false) {
			
		if (substr($_SERVER["REQUEST_URI"], 0, 10) == "/error.php") return;
		self::verifyIniFile();		
		
		// We only run this once..
		//if (isset($_SESSION["SYSTEMCHECK"])) return;	
		self::verifyDatabase($force);
		//$_SESSION["SYSTEMCHECK"] = true;
	}
	
	
	private function verifyIniFile() {
		$root = (is_null($_SERVER["DOCUMENT_ROOT"]) ? "." : $_SERVER["DOCUMENT_ROOT"]);
		$configFile = $root . "/../config/general.ini";
		$default = "title=Open iT Reporting Server\nserver=\nlogfile=\nloglevel=1\ndatabase=\ndbversion=2\ncharset=UTF-8\nlanguage=en\ntopbanner=\nbotbanner=\n";
		
		// If general.ini dosen't exist, we need to create one
		if (!file_exists($configFile)) {
			
			if (!$fh = fopen($configFile, 'a')) {
				header("Location: /error.php?error=" . self::ERROR_INI_FILE_CANT_OPEN);
				exit;
			}
			
			// Write $somecontent to our opened file.
			if (fwrite($fh, $default) === FALSE) {
				header("Location: /error.php?error=" . self::ERROR_INI_FILE_WRITE_ACCESS);
				exit;
			}

			fclose($fh);
		} else {
			
			// Check that we can update the file
			if (!is_writable($configFile)) {
				header("Location: /error.php?error=" . self::ERROR_INI_FILE_WRITE_ACCESS);
				exit;			
			}
			
		}
		
	}
	
	
	private function verifyDatabase($force) {
		self::updateVersion($force);
	}	
	
	
	private function updateVersion($force = false) {
		$version = ServerConfig::getDbVersion($force);
		
		if ($version != self::VERSION) {

			// We want to keep the user logged in, so we need to keep the object
			$user = isset($_SESSION["USER"]) ? $_SESSION["USER"] : null;
			
			// Destory session
			session_destroy();
			
			// Start new fresh session
			session_start();
			
			self::startUpdate();
			self::updateVersion_1($version);
			self::updateVersion_2($version);
			self::updateVersion_3($version);
			self::stopUpdate();			
			
			if (!is_null($user)) $_SESSION['USER'] = $user;
			
			if (!isset($_SERVER['SERVER_SOFTWARE'])) {			
				ob_end_clean();
			}
			
			exit;
		}
		
	}
	
	private function startUpdate() {
		// Make sure that the content is displayed on screen (as long as we are not in command-line mode)
		if (isset($_SERVER['SERVER_SOFTWARE'])) {
		
			// Load presentation layer class
			$presentation = new Presentation();
	
			// Show presentation
			$presentation->display("pages/update.php");
		
			ob_flush();
			flush();
		}		
	}
	
	private function stopUpdate() {
		if (isset($_SERVER['SERVER_SOFTWARE'])) {				
			$presentation = new Presentation();
			?>
				<script type="text/javascript" language="JavaScript">
					var updateing = document.getElementById("updating");
					updateing.innerHTML = "<?= htmlspecialchars($presentation->getText("updatemanager_done")) ?>";
					top.location = '/';
				</script>
			<?php
		}
	}
	
	// Default version
	private function updateVersion_1($version = null) {
		
		// Make sure we have the system database
		$dbRoot = ServerConfig::getDatabasePath() . "../general/";   //		$dbRoot = ServerConfig::getDatabasePath() . "trunk/src/general/";
		$dbFile = $dbRoot . "system.s3db";
		
		if (!file_exists($dbFile)) {
			new AccessControlManager(); // Generationg of the file happens here..
		}
		
	}
	
	
	/**
	 * Version 2
	 *  - New DB - settings for users
	 *  - INI changes: charset
	 */
	private function updateVersion_2($version = null) {
			if ($version < 2) {
				
				// Add new database table
				$dbRoot = ServerConfig::getDatabasePath() . "../general/";
				$dbFile = $dbRoot . "system.s3db";
				
				if (file_exists($dbFile)) {
					$db = new PDO("sqlite:" . $dbFile);
					$sql = file_get_contents($dbRoot . "sql/2.sql");
					$db->exec($sql);
					unset($db);
				}

				// Adding default charset
				ServerConfig::setCharset("UTF-8");
				
				// Update config file to be updated
				ServerConfig::setDbVersion(2);
			}
	}
	
	/**
	 * Version 3
	 *  - New DB - table version table
	 */
	private function updateVersion_3($version = null) {	
		if ($version < 3) {
				// Add new database table
				$dbRoot = ServerConfig::getDatabasePath() . "../general/";
				$dbFile = $dbRoot . "system.s3db";
				
				if (file_exists($dbFile)) {
					$db = new PDO("sqlite:" . $dbFile);
					$sql = file_get_contents($dbRoot . "sql/3.sql");
					$db->exec($sql);
					unset($db);
				}
				
				// Update config file to be updated
				ServerConfig::setDbVersion(3);			
		}
	}
}


// If running from command line, run the SystemChecker - but dont display data...
if (!isset($_SERVER['SERVER_SOFTWARE']) && !function_exists("__autoload2")) {
		
	function __autoload2($className) {
		$rootPath = (is_null($_SERVER["DOCUMENT_ROOT"]) ? "." : $_SERVER["DOCUMENT_ROOT"]) . '/../classes/';
	
		$filename = $rootPath . $className . '.php';
		if (file_exists($filename)) {
			require($filename);
			return;
		}
		
		// If we havent find the file here, we would need to check in sub folders
		$subFile = glob($rootPath . '*/' . $className . '.php');
		if (count($subFile) == 1) require($subFile[0]);
	
		// If we havent find the file here, we would need to check in sub folders
		$subFile = glob($rootPath . '*/*/' . $className . '.php');
		if (count($subFile) == 1) require($subFile[0]);
	
		// If we havent find the file here, we would need to check in sub folders
		$subFile = glob($rootPath . '*/*/*/' . $className . '.php');
		if (count($subFile) == 1) require($subFile[0]);
		
	}
	
	spl_autoload_register("__autoload2");
	
	SystemChecker::run();
	
}

?>
