<?php

define("TEMPLATELOADER_CACHE", "templateLoaderCache");
define("TEMPLATELOADER_CACHE_MD5", "templateLoaderCacheMD5");

define("TEMPLATEFOLDER_CACHE", "templateFolderCache");

class TemplateLoader {

	private $user;
	
	private $data;
	private $templates;
	private $md5;
	private $folderStatus;
	
	function __construct() {
		$this->load();
	}
	
	function __destruct() {
		$_SESSION[TEMPLATELOADER_CACHE_MD5] = $this->md5;
		$_SESSION[TEMPLATELOADER_CACHE] = $this->templates;
		$_SESSION[TEMPLATEFOLDER_CACHE] = $this->folderStatus;
	}
	
	private function load() {		
		
		$this->md5 = null;
		$this->templates = array();
		$this->folderStatus = array();
		$this->user = $_SESSION["USER"];
		
		if (isset($_SESSION[TEMPLATEFOLDER_CACHE])) {
			$this->folderStatus =& $_SESSION[TEMPLATEFOLDER_CACHE];
		}

		
		if (isset($_SESSION[TEMPLATELOADER_CACHE_MD5])) {
			$this->md5 =& $_SESSION[TEMPLATELOADER_CACHE_MD5];
		}

		if (isset($_SESSION[TEMPLATELOADER_CACHE])) {
			$this->templates =& $_SESSION[TEMPLATELOADER_CACHE];
		}

		// Get md5 check sum
		$this->data = file(ServerConfig::getServerUrl() . "simple_reports_dump.pl");
		$md5 = md5(implode("", $this->data));
			
		// Reload data
		if (is_null($this->md5) || empty($this->templates) || $this->md5 != $md5) {
			$this->md5 = $md5;
			$this->templates = array();
			$this->parse();
		}
		
	}

	private function parse() {
		
		$uid = 0;
		$mode = null;
		$templates = array();
					
		foreach ($this->data as $lineNumber => $line) {
			
		
								
		$TIME_START = microtime(true);
			
			$openitServerConfig = new OpeniT_Server_Config();
			$dataType = null;
			$line = trim($line);
			if (empty($line)) continue;
			
			// Find the DataType id (acctype)
			if (strpos($line, "acctype")) {
				$dataTypeId = substr($line, strpos($line, "acctype")+8);
				$dataTypeId = substr($dataTypeId, 0, strpos($dataTypeId, "&"));
			}

			if (!isset($dataTypeId) || !is_numeric($dataTypeId)) {
				Logger::Error("TemplateLoader::parse", "Can’t find DataType id in the URI - Continue with next line!", $lineNumber);
				continue;
			}
			
			
			// User access rights - Do the user have access to this data type?
			if (!$this->user->isAdmin() && !$this->user->isMemberOfDataType($dataTypeId)) {
				continue;
			}
			

			// Load data type
			$dataType = DatabaseManager::getDataType($dataTypeId);
			if (!$dataType) {
				Logger::Error("TemplateLoader::parse", "Can’t load DataType id with the ID " . $dataTypeId . " - SKIPPING", $lineNumber);
				continue;				
			}

			//Get the database path for checking of existence of data for the datatype use by the template
			$dbpath = $openitServerConfig->get('ROOT_DATA_DIR') . "/database/" . $dataTypeId;
			
			//If we don't have data for the datatype don't show the template
			if(!file_exists($dbpath)) {
				Logger::Debug("TemplateLoader::parse", "There is no data found for datatype {$dataTypeId} under {$dbpath}. The template will not be shown.");
				continue;
			}
			
			// Parse rest of the URI
			$params = explode("&", $line);
			foreach ($params as $param) {
				$param = urldecode($param);
				@list ($command, $value) = @explode("=", $param, 2);
				if (trim($value) == "") {
					Logger::Error("TemplateLoader::parse", "The command '" . $command .  "', doesn't have a value - skipping", $lineNumber);
					continue 1;
				}

				switch (strtoupper($command)) {
					
					case "COMMAND":
					case "ACCTYPE":
						break;
					
					// Select given measurements
					case "ACCVARS":
						$measurements = $dataType->getMeasurements();
						$items = explode(",", $value);
						foreach ($measurements as $measurement) {
							if (in_array($measurement->getId(), $items)) {
								$measurement->setSelected(true);
							}
						}						
						break;

					// Check that the user has access to the folder - Don't break! - We need to add it to the params
					case "FOLDER":
						if (!$this->user->isAdmin() && !$this->user->isMemberOfFolder($value)) {
							continue 3;
						}
						$dataType->reportData[$command] = $value;
						break;
						
					case "RESOLUTION":
						$dataType->reportData["resolution"] = $value;
						break;
						
					case "CLASSVARS":
						$classvars = explode(":", $value);
						foreach ($classvars as $id => $classvar) {
							$id++; // Since Time is the first one, we need to skip this one..
							if ($classvar != "x") {
								

								// Remove first and last char
								$classvar = substr($classvar, 1, -1);
								
								// Load classification
								$classification = $dataType->getClassificationByInternalId($id);
								
								// Verify that the classification is a valid
								if (is_null($classification)) {
									Logger::Error("TemplateLoader::parse", "Template is trying to get a classification set that doesn't match what we currently have in the data set. Skipping this template.", $lineNumber);
									continue 4;
								}

								// Mark classification as selected
								$classification->setSelected(true);
																
								// Check if the selection is controlled
								if ($classvar != "") {
									
									// check if the user can have any influence on the selection
									$controller = substr($classvar, 0, 1);
									$values = strlen($classvar) > 1 ? explode(",", substr($classvar, 2)) : array();
														
									if ($controller == "*") {
										$classification->allowInput = true;
										$classification->restricted = $values;
									}
									else if ($controller == "+") {
										$classification->allowInput = true;
										$classification->min = 1;
										$classification->restricted = $values;
									}
									else if ($controller == ".") {
										$classification->allowInput = true;
										$classification->min = 1;
										$classification->max = 1;
										$classification->restricted = $values;
									}
									else {
										$classification->allowInput = false;
										$values = explode(",", $classvar);
										$classification->restricted = $values;
										$classification->markAllClassificationValuesAsSelected();
									}
								} else {
									// Allow all
									$classification->allowInput = false;
								}
																
							}
						}
					
					default:
						$dataType->reportData[$command] = $value;
				}				
			}
		
			// Get a new uniq id
			$uid++;
			$dataType->reportData["uid"] = $uid;
			
			// Add the DataType as a report
			$dataType->refreshAccessRights();
			$this->templates[] = $dataType;		
			
		}

		// Sort the list by title
		usort($this->templates, array(new DataType(null, null, null, null, null), "sort"));
		
	}
	
	
	public function getTemplates() {
		return $this->templates;
	}
	
	
	/**
	 * Get available Templates for user, in a array designed
	 * for presentation in tree view.
	 *
	 * @param User $user Limit access to the templates based on the users access rights
	 * @return array $folders An array designed for tree view presentation
	 */
	public function getTemplateTreeByUser(User $user) {
		$folders = array();
	
		foreach ($this->templates as $template) {
			
			$folderNames = explode("/", $template->reportData["folder"]);
			$currentFolder =& $folders;
			
			foreach ($folderNames as $folderName) {
				if (!isset($currentFolder[$folderName])) {
					$currentFolder[$folderName] = array();
					$currentFolder[$folderName]["TEMPLATES"] = array();
				}
				$currentFolder =& $currentFolder[$folderName];
			}
			
			$currentFolder["TEMPLATES"][] = $template;
		}
		
		// Sort array with key as sort index
		ksort($folders);		
		
		return $folders;
	}
	
	public function getFolderStatus($folderName = null) {
		if (is_null($folderName)) {
			return $this->folderStatus;
		} else {
			if (isset($this->folderStatus[$folderName]) && $this->folderStatus[$folderName]) {
				return true;
			} else {
				return false;
			}
		}
	}

	public function setFolderStatus($folderName, $status) {
		$this->folderStatus[$folderName] = $status;
	}
	
	public function getTemplate($id) {

		for ($i=0; $i<count($this->templates); $i++) {
			if ($this->templates[$i]->reportData["uid"] == $id) {
				return $this->templates[$i];
			}
		}
		return;
		
		foreach ($this->templates as $template) {
			if ($template->getUid() == $id) {
				$template->load();
				return $template;
			}
		}		
		return false;
	}
	

}


?>