<?php


class TemplateSection {
	
	private $parent;
	
	private $title;

	private $acctype;
	private $classificationInternalId;
	private $values;	
	
	private $input;
	private $max;
	private $min;

	public $classification;	
	public $classificationValues;	
		
	function __construct(Template $parent, $acctype) {
		$this->parent = $parent;
		$this->acctype = $acctype;
		$this->values = array();
		$this->input = false;
		$this->min = 0;
		$this->max = -1;
	}
	
	public function load() {
		if (!$this->classification instanceof Classification) {
			$dataLoader = new DataLoader();
			$this->classification = $dataLoader->getClassificationForTemplateSection($this);
			//$this->classificationValues = $dataLoader->getClassificationsValuesForTemplateSection($this, $this->values);
		}
	}
	
	public function getAcctype() {
		return $this->acctype + 1;
	}
	
	public function setClassificationInternalId($id) {
		$this->classificationInternalId = $id;
	}

	public function getClassificationInternalId() {
		return $this->classificationInternalId + 1;
	}
	
	public function addValues($value) {
		if (is_array($value)) {
			if (count($value) == 0) return;
			$this->values = array_merge($this->values, $value);
		} else {
			if (trim($value) == "") return;
			$this->values[] = $value;
		}
	}
	
	public function getValues() {
		if (count($this->values) == 0) {
			$this->load();
		}
		return $this->values;
	}
	
	public function getClassification() {
		return $this->classification;
	}
	
	public function getClassificationValues() {
		$dataLoader = new DataLoader();		
		$this->classificationValues = $dataLoader->getValuesNewForTemplates($this->classification);
		
		/* mark selected */		
		$selectedValues = $this->classification->getSelectedValues();

		foreach ($this->classificationValues as $classificationValue) {
			if (in_array($classificationValue->getId(), $selectedValues)) {
				$classificationValue->setSelected(true);
			}
		}
		
		return $this->classificationValues;
	}
	
	public function getSelectedClassificationValues() {
		$classificationValues = array();
		foreach ($this->classificationValues as $classificationValue) {
			if ($classificationValue->isSelected()) {
				$classificationValues[] = $classificationValue;
			}
		}
		return $classificationValues;
	}
	
	public function numberOfSelections() {
		$classificationValues = $this->getSelectedClassificationValues();
		return count($classificationValues);
	}
	
	public function isSelectionValid() {
		$selections = $this->numberOfSelections();
		$max = $this->max;
		$min = $this->min;
		
		if (($max != -1 && $selections > $max) || ($selections < $min)) {
			return false;
		}
		
		return true;
	}
	
	public function setUserInput($boolean) {
		$this->input = $boolean ? true : false;
	}
	
	public function isUserInput() {
		return $this->input ? true : false;
	}
	
	public function setMax($max) {
		$this->max = $max;
	}
	
	public function getMax() {
		return $this->max;
	}
	
	public function setMin($min) {
		$this->min = $min;
	}

	public function getMin() {
		return $this->min;
	}
	
	public function isAnonymous() {
		if (isset($this->classificationValues[0])) {
			return $this->classificationValues[0]->isAnonymous();
		}
		return false;
	}

	public function isAnonymousFriendly() {
		if (isset($this->classificationValues[0])) {
			return $this->classificationValues[0]->isAnonymousFriendly();
		}
		return false;
	}
}

?>