<?php

class User {
	
	// Error
	const ERROR_USERNAMEEXISTS = 10;
	const ERROR_NONAMEORNOUSERNAME = 20;
	
	private $id;
	private $username;
	private $password;
	private $name;
	private $division;
	
	// Roles
	private $admin;
	private $agreement;
	private $templateUser;
	private $viewUser;
	private $reportGeneratorUser;

	private $groups;
	private $folders;
	
	public function __construct($id = null, $username = null, $password = null, $name = null, $division = null, $admin = null, $agreement = null, $templateUser = null, $viewUser = null, $reportGeneratorUser = null) {
		$this->id = $id;
		$this->username = $username;
		$this->password = $password;
		$this->name = $name;
		$this->division = $division;

		// Roles
		$this->admin = $admin;
		$this->agreement = $agreement;
		$this->templateUser = $templateUser;
		$this->viewUser = $viewUser;
		$this->reportGeneratorUser = $reportGeneratorUser;

	}
	
	
	public function getId() {
		return $this->id;
	}
	
	public function getUsername() {
		return $this->username;
	}
	
	public function getPassword() {
		return $this->password;
	}
	
	public function getName() {
		return $this->name;
	}
	
	public function getDivision() {
		return $this->division;
	}
	
	public function isAdmin() {
		return $this->admin ? true : false;
	}
	
	public function isAgreement() {
		return $this->agreement ? true : false;
	}
	
	public function isTemplateUser() {
		return $this->templateUser ? true : false;
	}
	
	public function isViewUser() {
		return $this->viewUser ? true : false;
	}
	
	public function isReportGeneratorUser() {
		return $this->reportGeneratorUser ? true : false;
	}
	
	public function isRealtimeUser() {
		return UserSettings::getValue($this, UserSettings::REALTIME_ACCESS) ? true : false;
	}
	
	public function isMemberOfDataType($datatypeId) {
		if (empty($this->groups) && $this->id != "") {
			$this->groups = AccessControlManager::getUserMembers($this->id, true);
		}
		
		foreach ($this->groups as $group) {
			if ($group->isMemberOfDataType($datatypeId)) return true;
		}
		
		return false;
	}
	
	public function isClassificationAnonymous($dataTypeId, $classificationId, $folder = null) {
		if ($this->admin) return false;
		
		if (empty($this->groups) && $this->id != "") {
			$this->groups = AccessControlManager::getUserMembers($this->id, true);
		}

		$anonymous = null;
		foreach ($this->groups as $group) {

			if (!$group->isMemberOfDataType($dataTypeId)) {
				Logger::Debug("User::isClassificationAnonymous()", $group->getName() . " is not member of datatype with id " . $dataTypeId);
				continue;	
			}
			
			if (!is_null($folder)) {
				if ($group->isMemberOfFolder($folder) && (is_null($anonymous) || $anonymous == true)) {
					if ($group->isClassificationAnonymous($dataTypeId, $classificationId)) {
						return true;
					}
				}
			} else {
				if ($group->isClassificationAnonymous($dataTypeId, $classificationId) && (is_null($anonymous) || $anonymous == true)) {
					return true;
				}
			}
		}
		
		return false;
	}
	
	public function hasValueFiltered($clsfParent, $clsfValue) {
		if ($this->admin) return false;
		
		if ($this->id != "") {
			$this->groups = AccessControlManager::getUserMembers($this->id, true);
		}
		
		foreach ($this->groups as $group) {

			if ($group->isMemberOfProduct($clsfValue)&& $clsfParent=="Product") {
				return true;	
			}else if ($group->isMemberOfFeature($clsfValue)&& $clsfParent=="Feature") {
				//Logger::Debug(__CLASS__.'::'.__METHOD__, "The feature " . $clsfValue . " has been hidden from " . $group->getName());
				return true;
			}
			
		}
		
		return false;
	}
	
	public function setId($id) {
		$this->id = $id;
	}

	public function setUsername($username) {
		$this->username = $username;
	}
	
	public function setPassword($password) {
		$this->password = md5($password);
	}
	
	public function setName($name) {
		$this->name = $name;
	}
	
	public function setDivision($division) {
		$this->division = $division;
	}
	
	public function setAdmin($admin) {
		$this->admin = $admin ? 1 : 0;
	}

	public function setAgreement($agreement) {
		$this->agreement = $agreement ? 1 : 0;
	}

	public function setTemplateUser($templateUser) {
		$this->templateUser = $templateUser ? 1 : 0;
	}
	
	public function setViewUser($viewUser) {
		$this->viewUser = $viewUser ? 1 : 0;
	}
	
	public function setReportGeneratorUser($reportGeneratorUser) {
		$this->reportGeneratorUser = $reportGeneratorUser ? 1 : 0;
	}
	
	public function setRealTimeUser($realtimeUser) {
		// if user is not stored yet, we need to save it before we set the realtime user properties
		if (is_null($this->id)) {
			if (!$this->store()) {
				Logger::Critical(__CLASS__ . '::setRealTimeUser', 'Could not add new user to the RealTime role - Please update the user manually!');				
			} 
		}
		UserSettings::setValue($this, UserSettings::REALTIME_ACCESS, $realtimeUser ? 1 : 0);
	}
	
	public function getGroups() {
		if (empty($this->groups) && $this->id != "") {
			$this->groups = AccessControlManager::getUserMembers($this->id, true);
		}
		return $this->groups;
	}
	
	public function isMemberOf($group) {
		if ($this->id == "") return array();
				
		if (empty($this->groups)) {
			$this->groups = AccessControlManager::getUserMembers($this->id);
		}
		
		if (in_array($group, $this->groups)) {
			return true;
		}
		
		return false;
	}
	
	public function store() {
		return AccessControlManager::updateUser($this);
	}
	
	public function getFolders() {
		
		// If admin, we get all folders
		if ($_SESSION["USER"]->isAdmin()) {
			$this->folders = FolderManager::getFolders();
			return $this->folders;
		}
		
		// .. if not, we need to check if the user has access
		// to the given folders..
		$this->folders = array();
		$groups = $this->getGroups();
		foreach ($groups as $group) {
			$folders = $group->getFolders();
			foreach ($folders as $folder) {
				if (!in_array($folder, $this->folders)) {
					$this->folders[] = new Folder($folder);
				}
			}
		}
		return $this->folders;
		
	}
	
	public function isMemberOfFolder($folder) {
		 if ($folder instanceof Folder) $folderName = $folder->getName();
		 else $folderName = $folder;
		 
		 foreach ($this->getFolders() as $folder) {
		 	if ($folder->getName() == $folderName) return true;
		 }
		
		 return false;	
	}
	
	public function getLanguage() {
		return UserSettings::getValue($this, UserSettings::LANGUAGE);
	}
	
	public function setLanguage($language) {
		// if user is not stored yet, we need to save it before we set the realtime user properties
		if (is_null($this->id)) {
			if (!$this->store()) {
				Logger::Critical(__CLASS__ . '::setLanguage', 'Could not set language for new user - Please update the language manually!');				
			} 
		}		
		UserSettings::setValue($this, UserSettings::LANGUAGE, $language);
	}
	
}

?>