<?php

/**
 * UserSettings - Interface to the UserSettings table
 */
class UserSettings {
	
	// All values that could be set as a key in UserSettings, should exists as a constant here
	const LANGUAGE = "language";
	const REALTIME_ACCESS = "realtime_access";

	
	// Static link with the database resource
	static private $db;
	
	
	/**
	 * Internal function for checking if there is a database resource created
	 */
	static private function dbcheck() {
		// Create a link to the database
		if (self::$db == null) {
			$dbRoot = ServerConfig::getDatabasePath() . "../general/";
			$dbFile = $dbRoot . "system.s3db";
		
			// This should not happen
			if (!file_exists($dbFile)) {
				return null;
			}
			
			// Save the link
			self::$db = new PDO("sqlite:" . $dbFile);
		}
	}
	

	/**
	 * Returns data from UserSettings database
	 *
	 * @param User $user Which user to get data from
	 * @param String $key Key for the value
	 * @return String
	 */
	static function getValue(User $user, $key) {
		
		self::dbcheck();
		
		$sql = "SELECT value FROM UserSettings WHERE user = " . $user->getId() . " AND key = '" . str_replace("\"", "'", $key) . "'";
		foreach (self::$db->query($sql) as $row) {
			return $row[0];
		}
		
		return null;
	}
	
	
	/**
	 * Function that set data to UserSettings
	 *
	 * @param User $user User that should have information added/updated
	 * @param String $key Key for the value
	 * @param String $value The value that should be set
	 */
	static function setValue(User $user, $key, $value) {
		self::dbcheck();
		
		// Check if data exists
		if (self::getValue($user, $key) == null) {
			$sql = "INSERT INTO UserSettings VALUES (null, " . $user->getId() . ", '" . escape_string($key) . "', '" . escape_string($value) . "')";
		} else {
			$sql = "UPDATE UserSettings SET value = '" . escape_string($value) . "' WHERE user = " . $user->getId() . " AND key = '" . escape_string($key) . "'";
		}
		
		self::$db->query($sql);
	}
	
}

?>