<?php

class XMLParser {

	public static function xmlToObj(&$content, &$xml = null, $tagName = null) {
		
		// If we don't, have a tagName this is the first time we run this methode,
	 	// and we need to find the root element of this XML, and replace the content
		// with the complete root element 
		if (is_null($xml)) {
			preg_match("#<([a-z]+)( .*)?>#imsU", $content, $out);
			preg_match_all("#<" . $out[1] . "( .*)?" . ">(.*)</" . $out[1] . ">#ims", $content, $out);
			$content = $out[2][0];
			$xml = new XMLNode();
		}
		
		// Loop trhu content, as long as we have something..
		while (trim($content) != "") {
			
			// Find next tag
			//preg_match("#(.*)<([/]?)([a-z]+)( .*)?([/]?)>#imsU", $content, $out);
			$out = self::getNextTag($content); // Instead of using regular expression (above), we try this one - since it is a bit faster..
			
			// No data returned, return blank
			if (!isset($out[4])) return;
			
			// Set variables
			$tagContent 	= str_replace(array('<![CDATA[', ']]>'), '', $out[0]);	// Remove CDATA tag
			$tagValue 		= trim($out[1]);		
			$closingTag 	= trim($out[2]) != "" ? true : false; 
			$tag 			= $out[3];
			$tagAttributes 	= trim($out[4]);
			$shortTag 		= trim($out[5]) != "" ? true : false;	
			$openTag 		= !$closingTag && !$shortTag ? true : false;
	
			// Remove from content
//			$content = str_replace_once($tagContent, "", $content);
					
			// Are we closing an open tag, if so, return
			if ($closingTag && $tag == $tagName) {
				if ($tagValue != "") $xml->CDATA = $tagValue;
				return;		
			}		
	
			// Create place holder for the node(s)
			//if (!isset($xml->$tag) || !is_array($xml->$tag)) $xml->$tag = array();
			if (!isset($xml->$tag)) $xml->$tag = array();
			//if (is_null($xml->$tag)) $xml->$tag = array();
			
			// Get the reference to the place holder
			$xmlNode =& $xml->$tag;
			
			// Create new node
			$node = new XMLNode();
			$node->_NAME = $tag;
			
			// Attributes
			if (($openTag || $shortTag) && $tagAttributes != "") {
				preg_match_all("#\s*(\w*)=\"(.*)\"#imsU", $tagAttributes, $tagAttributeSets);
				foreach ($tagAttributeSets[1] as $tagAttributeSetId => $key) {
					if (!isset($node->_ATTRIBUTES)) $node->_ATTRIBUTES = array();
					$node->_ATTRIBUTES[] = new XMLAttribute($key, html_entity_decode(str_replace('"', "", $tagAttributeSets[2][$tagAttributeSetId])));
				}					
			}						
			
			// If its a opentag, we need to get deeper..
			if ($openTag) XMLParser::xmlToObj($content, $node, $tag);
			
			// Set node to tree
			$xmlNode[] = $node;								
		}
		
		return $xml;	
	}
	
	static private function getNextTag(&$content) {
		// Vars
		$EOF = false;
		$pos = 0;
		$contentLength = strlen($content);
		$out = array(0 => '', 1 => '', 2 => '', 3 => '', 4 => '', 5 => '');
		$inTag = false;
		$hasTagName = false;
		
		do {
		
			// Get current char
			$char = $content[$pos];
			
			// Prepeare to go to next position
			$pos++;
			if ($pos >= $contentLength) $EOF = true;
			
			// complet string
			$out[0] .= $char;
			
			// Start of tag..
			if ($char == '<') {
				$inTag = true;
				continue;
			}
			
			// End of tag, we are parsing..
			else if ($char == '>') {
				break;
			}
			
			// Not in tag, got data.. part of text
			else if (!$inTag) {
				$out[1] .= $char;	
			}
			
			// In tag, got /, and dosent have tagname.. closing tag
			else if ($inTag && $char == '/' && strlen($out[3]) == 0) {
				$out[2] = true;
			}	
				
			// In tag, got / and has tag chars.. short tag
			else if ($inTag && $char == '/' && strlen($out[3]) != 0) {
				$hasTagName = true;
				$out[5] = true;
			}	
			
			// In tag, got data.. has tag name, part of attribute
			else if ($inTag && $hasTagName) {
				$out[4] .= $char;
			}
			
			// In tag, got ' ' .. end of tagName
			else if ($inTag && $char == ' ') {
				$hasTagName = true;
			}
			
			// In tag, got data.. part of tagName
			else if ($inTag) {
				$out[3] .= $char;
			}
		
			
		} while (!$EOF);
		
		// remove from content
		$content = substr($content, strlen($out[0]));
		
		return $out;
	}	
}

class XMLNode {
	
	public function get($key) {
		if (!isset($this->_ATTRIBUTES)) {
			return null;		
		}		
		foreach ($this->_ATTRIBUTES as $attribute) {
			if ($attribute->key == $key) {
				return $attribute->value;
			}
		}
		return null; 
	}
	
	public function next() {
		if (!isset($this->_ATTRIBUTES)) {
			return null;		
		}
		
		$current = current($this->_ATTRIBUTES); 
		next($this->_ATTRIBUTES);
		return $current; 
	}
	
	public function reset() {
		return !isset($this->_ATTRIBUTES) ? null : reset($this->_ATTRIBUTES);
	}
	
	public function hasMore() {
		if (!isset($this->_ATTRIBUTES)) {
			return null;
		}
		
		$hasMore = true;
		next($this->_ATTRIBUTES);
		if (key($this->_ATTRIBUTES) === null) $hasMore = false;
		prev($this->_ATTRIBUTES);
		
		return $hasMore; 		
	}
	
	public function count() {
		return !isset($this->_ATTRIBUTES) ? null : count($this->_ATTRIBUTES);
	}
	
	public function getText() {
		return isset($this->CDATA) ? $this->CDATA : null;
	}
	
	public function children() {
		$elements = array();		
		foreach ($this as $element) {			
			if (is_array($element) && ($element[0] instanceof XMLNode)) {
				$elements = array_merge($elements, $element);
			}
		}		
		return $elements;
	}
	
	public function getName() {
		return $this->_NAME;
	}
	
}

class XMLAttribute {
	public function __construct($key, $value) {
		$this->key = $key;
		$this->value = $value;
	}
	public $key;
	public $value;	
}


function str_replace_once($search, $replace, $subject) {
    if (($pos = strpos($subject, $search)) !== false) {
        return substr($subject, 0, $pos) . $replace . substr($subject, $pos + strlen($search));
    }    
    return $subject;
}

?>