<?php


class wgDatabaseTable extends wgDatabaseQuery {
	
	public function __construct($class, $table, $database = null) {
		if (!is_null($database)) {
			die('Does not support anything else than default database...');
		}
		
		$this->className = $class;
		$this->tableName = $table;
				
		// Verify database
		wgDatabaseVerify::verify($this->tableName);
		
	}
		
	/**
	 * @return wgDatabaseSelect
	 */
	public function select($fields = '*') {
		$this->addSQL('SELECT ' . $fields . ' FROM ' . $this->tableName); 
		return new wgDatabaseSelect($this);
	}
	
	/**
	 * @return wgDatabaseDelete
	 */
	public function delete() {
		$this->addSQL('DELETE FROM ' . $this->tableName); 
		return new wgDatabaseDelete($this);
	}
	
	/**
	 * @return wgDatabaseUpdate
	 */
	public function update() {
		$this->addSQL('UPDATE ' . $this->tableName); 
		return new wgDatabaseUpdate($this);
	}
	
	public function store($data, &$lastInsertId = null) {

		// Convert object to array
		if (is_object($data)) {
			$mData = new ClassModifyer($data);
			$data = $mData->getValues();
		}
		
		// Get database
		$pdo = $this->getDatabase();
		
		// Get database fields
		$fields = array();
		$result = $pdo->query("pragma table_info ('{$this->tableName}');");
		$rows = $result->fetchAll(PDO::FETCH_ASSOC);
		$pk = null; // primary key
		$pkValue = null; // primary key value
				
		// Create a nice array with all fileds
		foreach ($rows as $row) {
			if (is_null($pk) && $row['pk'] == '1') $pk = $row['name'];  
			$fields[] = $row['name'];
		}

		// Go thru the object we want to store
		$keys = array();		
		foreach ($data as $key => $value) {
		
			// Check if we have the key as a field
			if (!in_array($key, $fields)) continue;
			
			// See if its a index, we keep the value
			if ($key == $pk) $pkValue = $value;			
			
			// Add key to local variable
			$keys[] = '"'.$key.'"';
			$this->addValue($value);			
		}
		
		// Check if we have something to save..
		if (count($keys) == 0) {
			throw new Exception('No data to save..');
			return;
		}
		
		// If PK is set, then its an update
		if (!is_null($pk) && !is_null($pkValue)) {
			$this->addValue($pkValue);
			$this->addSQL("UPDATE {$this->tableName} SET " . (implode(' = ?, ', $keys)) . " = ? WHERE \"{$pk}\" = ?;");
		}
		
		// No PK, so its a new insert
		else { 	
			// Build and add SQL		
			$this->addSQL("INSERT INTO {$this->tableName} (" . (implode(', ', $keys)) . ") VALUES (?" . str_repeat(', ?', count($keys)-1)  .  ");");
		}
				
		$return = new wgDatabaseReturn($this);
		return $return->execute($lastInsertId);		
	}
	
	
	
	private function fetchEntry() {
		return get_class(self);		
	}
	
	private function fetchEntries($table) {
		return new wgDatabaseSelect();
	}
	
}


?>