<?php

/**
 * Verify that the database tables are uptodate
 */
class wgDatabaseVerify {
	
	static private $SESSION_KEY = 'wgDatabaseVerify';
	
	static public function verify($table) {
		
		// Make sure that session holder, is an array
		if (!isset($_SESSION[self::$SESSION_KEY]))
			$_SESSION[self::$SESSION_KEY] = array();  

		// don't check tableversion
		if ($table == 'tableversion') return true;
		
		// If table is in session list, we dont check it..
		if (in_array($table, $_SESSION[self::$SESSION_KEY])) return;
		
		// Get files with latest version for table
		$versions = self::getVersionList($table);	
		$currentVersion = wgDbTableVersion::getVersion($table);
		
		// If it doesnt exists, create all
		if (is_null($currentVersion)) {
			foreach ($versions as $version) {
				self::runDatabaseScript($table, $version);
			}
		}
		
		// Run new database scripts 
		else {

			// Get versionumber for object
			$currentVersion = $currentVersion->version;
			
			// Check if there is something newer...
			foreach ($versions as $version) {
				// Check if we need to update our table by running SQL update scripts..
				if ((int) $currentVersion < (int) $version) {			
					self::runDatabaseScript($table, $version);
				}
			}
		}
		
		// Add to session array
		$_SESSION[self::$SESSION_KEY][] = $table;
	}
	
	
	private static function runDatabaseScript($table, $version) {
		
		// Log information
		ConsoleLogger::write("Updating {$table} to version {$version}");
		Logger::Info(__CLASS__, "Updating {$table} to version {$version}");
		
		// Run SQL script for given table and version number
		$dbRoot = ServerConfig::getDatabasePath() . "../general/";
		$dbFile = $dbRoot . "system.s3db";
		
		$sqlFile = $dbRoot . "sql/tables/" . $table . "_" . $version . ".sql";
		
		if (file_exists($dbFile) && file_exists($sqlFile)) {
			$db = new PDO("sqlite:" . $dbFile);
			$sql = file_get_contents($sqlFile);
			$db->exec($sql);
			unset($db);
		}
		
		// Update database..
		wgDbTableVersion::set($table, $version);
	}
	
	private static function getVersionList($table) {		
		$versions = array();		
		$files = glob(ServerConfig::getDatabasePath() . "../general/sql/tables/" . $table . "_*.sql");		
		foreach ($files as $file) {
			list($table, $version) = explode("_", basename($file, '.sql'));
			$versions[] = $version;
		}
		sort($versions);
		return $versions;
	}
	
}

?>