<?php

/**
 * 
 * GuiMapping
 * 
 * Used by GUI to build a virtual classification.
 * Can be used to simplyfi the selection of other classifications
 *
 */
class GuiMapping {
	
	
	/**
	 * Comments starts with #
	 * 
	 * Key word is: "MAP: " connected with the combind name e.g. "MAP: Geoframe"
	 * Every line after this keyword, contains of a product name that will be mapped
	 * to this combind name.
	 * 
	 * Product names can use wildcars '?' to match any character or 
	 * '*' to match any substring (0 or more characters).
	 * 
	 */
	
	private $file;
	
	private $label;
	private $datatypes;
	private $map;
	
	
	public function GuiMapping($filname = null) {
		
		if (is_null($filname)) {
			throw new Exception("We need a product map file..");
		}

		/*if (is_file($filname)) {
			throw new Exception("Could not find map file {$filname}");
		}*/
		
		$this->map = array();
		$this->file = $filname;
		
		$this->load();
	}
	
	private function load() {
		
		// Parse the mapping file, to an easier format for looking up data..
		$parser = new Parser(file_get_contents($this->file),
			ParserSettings::create(ParserSettings::COMMENT_CHAR, '#')
			->set(ParserSettings::SKIP_COMMENTS, true)
			->set(ParserSettings::SKIP_BLANK_LINES, true)
		);
		
		$label = null;
		$classification = null;
		$datatype = null;
		
		// For each line we have...
		while ($parser->hasMore()) {
			
			// Get next line
			$line = $parser->next();
			
			$keyword = strstr(trim($line->toString()), ":") ? substr(trim($line->toString()), 0, strpos(trim($line->toString()), ':')) : null;			
			switch ($keyword) {
				case "MAP":
					try {
						$map = $line->next(ParserSettings::create(ParserSettings::SPLIT, ParserSettings::SPLIT_TYPE_SEMI_COLON));
						$mapdetails = $map->get(1)->next(ParserSettings::create(ParserSettings::SPLIT, ParserSettings::SPLIT_TYPE_PARENTESE_START));				
						$label = trim($mapdetails->get(0)->toString());						
						//$datatype = $mapdetails->get(1, (ParserSettings::create(ParserSettings::SPLIT, ParserSettings::SPLIT_TYPE_SEMI_COLON)))->get(0)->toString();						
						$classification = str_replace(')', '', trim($mapdetails->get(1)->toString()));

						//if (!empty($datatype) && !empty($classification)) {						
						if (!empty($classification)) {
							$this->map[$label] = array();
							$this->map[$label]['CLASSIFICATION'] = $classification;
							//$this->map[$label]['DATATYPE'] = $datatype;
							$this->map[$label]['VALUES'] = array();
						}
						
					} catch (Exception $e) {
						Logger::Debug(__FILE__, "Error parsing virtual gui file.. ({$this->file}), error: $e", __LINE__);
					}					
					continue 2;
					
				case "LABEL":
					$this->label = trim(substr($line->toString(), strpos($line->toString(), ':')+1));
					continue 2;
					
				case "DATATYPES":
					$this->datatypes = trim(substr($line->toString(), strpos($line->toString(), ':')+1));
					continue 2;
					
				case "CLASSIFICATION":
					continue 2;
			}
									
			// if we dont have a product name, we ontinue..
			//if (empty($datatype) || empty($classification)) continue;
			if (empty($classification)) continue;
		
			// add product name to combined product name
			$this->map[$label]['VALUES'][] = trim($line->toString());
		}
		
	}
	
	public function getMap() {
		return $this->map;
	}
	
	public function getLabel() {
		return $this->label;		
	}
	
	public function getDatatypes() {
		return $this->datatypes;		
	}
	
	
	public function getParent($productName) {		
		foreach ($this->map as $combinedName => $values) {
			foreach ($values as $value) {
				if (trim(strtolower($productName)) == trim(strtolower($value))) {
					return $combinedName;
				}
			}
		}
		return null;		
	}
	
	
}

?>