<?php

class GuiMappingManager {
	
	const CONFIG_PATH = '/var/opt/openit/etc/favorites/';
	
	private $db;
	
	public function __construct() {
		$this->db = new PDO("sqlite:" . ServerConfig::getDatabase());		
		if (!$this->db) {
			Logger::Error("GuiMappingManager:__constructor", "Can’t load the database file! Stopping program!!");
		}		
	}
	
	
	public function load($file) {
		if (!$this->db) return;
		
		$map = new GuiMapping($file);
		$label = $map->getLabel();
		$datatypeIds = (strstr($map->getDatatypes(), ',')) ? explode(',', $map->getDatatypes()) : array($map->gwqetDatatypes());
		
		foreach ($datatypeIds as $datatypeId) {

			// Verify that datatypeid is nummeric
			if (!is_numeric($datatypeId)) {
				Logger::Error(__FILE__. "Datatype is not nummeric, skipping... ({$file})");
				continue;
			}
		
			// Create Virtual Classification
			$sql = 'INSERT INTO Classification VALUES (null, ' . $datatypeId . ', (SELECT count(*)+1 FROM classification WHERE datatypeId = ' . $datatypeId . '), "' . $label . '", 0, 0, 1);';
			$classificationId = '(SELECT id FROM classification WHERE datatypeId = ' . $datatypeId . ' AND value = "' . $label .'")';
					
			
			// For each MAP in the config
			foreach ($map->getMap() as $virtualLabel => $virtualMap) {
	
				// Set label
				$sql .= 'INSERT INTO classificationvalue VALUES (null, ' . $classificationId . ', "'. $virtualLabel . '", null, 0);';
								
				// Classification to map to
				$classificationSource = $virtualMap['CLASSIFICATION'];
				
				$newCV = '(SELECT id FROM classificationvalue WHERE classificationId = ' . $classificationId . ' AND value = "'. $virtualLabel . '")';
				$classificationSourceId = '(SELECT id FROM classification WHERE datatypeId = ' . $datatypeId . ' AND value = "' . $classificationSource . '")';
				
				// Classification values to link to
				foreach ($virtualMap['VALUES'] as $classificationSourceValue) {
	
					// Find classification soruce id in existing dataset..
					$origCV = '(SELECT id FROM classificationvalue WHERE classificationId = ' . $classificationSourceId . ' AND value = "'. $classificationSourceValue . '")';
							
					// Create entries in valuesrelationship 
					$sql .= 'INSERT INTO valuesrelationship VALUES (' . $newCV . ', ' . $origCV . ');';						
				}		
				
			}
			
			//echo $sql;
			if ($this->db->exec($sql)) {
				Logger::Debug(__FILE__, "Successfully loaded the Virtual Gui Classification file {$file} for datatype id {$datatypeId}");
			} else {
				Logger::Critical(__FILE__, "Can't load Virtual Gui Classification file {$file} for datatype id {$datatypeId}, something failed... Please check syntax in map file {$file}");
			}
			
		}
		
	}
	
	public static function run() {
		
		// Log message
		Logger::Debug(__FILE__, 'Running Virtual Gui Classifcation manager...');
		
		// Find each file in the given folder
		$files = glob(self::CONFIG_PATH . '*.map');
		
		if (count($files) == 0) {
			Logger::Debug(__FILE__, "There is no Virtual Gui Classification maps on the system...");
			return;
		}
		
		$guiMapping = new GuiMappingManager();
		
		// Add each file under the given folder..
		foreach ($files as $file) {
			$guiMapping->load($file);	
		}
		
	}
	
	
}

?>