<?php

abstract class AbstractController implements Controller {
	
	private $document;	
	private $request;
	private $moduleInfo;
	private $variables;
	
	private $validator;
	
			
	final public function __construct($request) {
		$this->document = new stdClass();
		
		$this->document->title = '';
		$this->document->template = 'normal';
		
		$this->document->message = new stdClass();
		$this->document->message->type = '';
		$this->document->message->text = '';
		
		
		$this->document->head = '';
		$this->document->body = '';
		$this->document->menu = null;		
		$this->document->subMenu = null;
		
		$this->document->scripts = array();
		$this->document->styles = array();
		
		// Set request object		
		$this->request = $request;
		
		// Set class info
		$this->moduleInfo = new ModuleInfo(get_class($this));
		
		$this->variables = array();
		
		// Calls init in construct fase
		$this->init();		
	}
	
	/**
	 * Empty function
	 * 
	 * Created here, so the user module can override this one, to create their own construct.
	 * This is called from the AbstractController construt function (when class is initialized)
	 *
	 */
	public function init() {}
	
	public final function getDocument() {
		return $this->document;
	}
	
	public final function addVariable($key, $value) {
		$this->variables[$key] = $value;
	}
	
	public final function getVariables() {
		return $this->variables;
	}
	
	public final function getVariable($key) {
		return isset($this->variables[$key]) ? $this->variables[$key] : null;
	}
	
		
	/**
	 * @return Request2
	 */
	public final function getRequest() {
		return $this->request;
	}
	
	/**
	 * Enter description here...
	 *
	 * @return Validator
	 */
	public final function getValidator() {
		if (is_null($this->validator)) {
			$this->validator = new Validator($this->getRequest()->_REQUEST);
		}
		return $this->validator;
	}
	
	public final function redirect($uri) {
		header('Location: ' . $this->getURL($uri));
	}
	
	public final function forward($uri) {
		
	}
	
	
	public function getMenu() {
		return null;
	}
	
	public function getSubMenu() {
		return null;
	}
	
	
	/**
	 * Return data from rendering
	 *
	 * @param String $template
	 * @param Mixed $params
	 * @return String Data from rendering
	 */
	public final function render($template, $params = array()) {
		$_template =  $this->getModuleInfo()->getModulePath() . FrontController::MODULE_FOLDER_HTML . $template . '.php';
		if (file_exists($_template)) {
			
			foreach ($params as $_key => $_value) {
				${$_key} = $_value;
			}
			
		// Local variables
			$list = $this->getList();
			$model = $this->getModel();
			$controller = $module = $this;
			$document = $this->getDocument();
			
			
			ob_start();
			include $_template;
			$data = ob_get_contents();
			ob_end_clean();
			return $data;			
		}
		return '';
	}
	
	public function getSubModules() {
		return array();
	}
	
	
	/**
	 * Return classname
	 *
	 * @return String class name
	 */
	public function getClassName() {
		return get_class($this);
	}
	
	/**
	 * Generate URL
	 *
	 * @param unknown_type $subs
	 * @return String URL
	 */
	public final function getURL($subs = null) {
		
		$actions = Request::getActions();
		
		// Build an array 
		if (is_null($subs)) {
			$subs = array();
		} else if (!is_array($subs)) {
			$subs = array($subs);
		}
		
		// Replace null variables in array, with values from existing action
		foreach ($subs as $index => $sub) {
			if (is_null($sub) && isset($actions[$index+1])) {
				$subs[$index] = $actions[$index+1];
			}						
		}
				
		$url = array_merge(array('/' . RequestHandler::WRAPPER, $actions[0]), $subs);
		return implode('/', $url);
	}
	
	public function setMessage($message, $type = null) {
		$this->document->message->text = $message;
		if (!is_null($type)) {
			$this->document->message->type = $type;
		}
	}
	
	
	/**
	 * Return information about this module
	 *
	 * @return ModuleInfo
	 */
	public final function getModuleInfo() {
		return $this->moduleInfo;
	}
	
}

?>