<?php

/**
 * 
 * This is the very first controller that handles the request * 
 *
 * @static 
 */
class FrontController {
	
	const PRESENTATION_FOLDER_TEMPLATES = '../modules/html/';
	
	// Each module class starts with this extension
	const MODULE_CLASS_DEFINITION = 'Mod';
	
	const MODULE_FOLDER_HTML = 'presentation/';
	const MODULE_FOLDER_STATIC = 'docs/';
	
	
	// Start front controller
	static public function init() {
		
		// debug
		if (SystemSettings::get(SystemSettings::SESSION_DEBUG)) {
			echo '<pre>';
			print_r(RequestParser::getBaseUri());
			print_r(RequestHelper::getModule());
			ini_set('display_errors', true);
			echo '</pre>';
		}
			
		// Get controller
		try {
			$module = RequestHelper::getModule($action);
			self::executeController($module);
		} catch (Exception $e) {
			echo 'Error handeling first controller...' . $e;
		}
		
	}
	
	static private function executeController(AbstractController $module) {
		
		
		// Always call the init script
		$module->init();
		
		$action = $module->getRequest()->action;
		
		// debug
		if (SystemSettings::get(SystemSettings::SESSION_DEBUG)) {
			var_dump($module);
			echo 'action: ' . $action;
		}
		
		// Check for static files, show contents, if exists..
		if ($module->getRequest()->file != '' && file_exists($module->getModuleInfo()->getModulePath() . self::MODULE_FOLDER_STATIC . $module->getRequest()->file)) {
			echo file_get_contents($module->getModuleInfo()->getModulePath() . self::MODULE_FOLDER_STATIC . $module->getRequest()->file);
			return;			
		}
		
		
		// Need validate
		if (($action == "update" || $action == "put") && method_exists($module, 'validate')) {

			
			// Check if document validates
			ob_start();
			$validate = $module->{'validate'}();
			ob_end_clean();
			
			// If its null, we get the value directly from the validator object
			if (is_null($validate)) {
				$validate = $module->getValidator()->validate();
			}
			
			// Does not validte
			if (!$validate) {
				
				var_dump($validate);
				var_dump($module->getValidator()->getErrors()); 
				
				// We would need to go back to last page, and allow user to change the data

				die('validation failed..');
				exit;
			}
			
		}
		
		// Enable tt function
		if (!function_exists('tt')) {
			function tt($key) {
				$ini = parse_ini_file(ServerConfig::getLanguagePath() . strtolower(ServerConfig::getCharset()) . "/" . ServerConfig::getDefaultLanguage() . ".ini");
				$string = isset($ini[$key]) ? $ini[$key] : "NO KEY WITH : $key";
			
				for ($i=1; $i<func_num_args(); $i++) {
					$value = func_get_arg($i);
					$string = str_replace("%" . $i, $value, $string);
				}
			
				return $string;					
			}
		}
		

		// Get the ouput from the controller class,
		// while executing the methode
		ob_start();
		$result = $module->{$action}();
		$data = ob_get_contents();
		ob_end_clean();
		
		
		// If this is an validation, we could stop here...
		if ($action == 'validate') {
			
			// If its valid, we could stop here..
			if ($result === true || $module->getValidator()->validate()) {
				echo 'true';
				return;				
			}
			
		
			// Get validator
			$data = $module->getValidator()->getErrors();
		}

		
		// Presentation variable
		$presentation = null;
		
		// Check the extension, if we should call a specific output filter..
		switch (strtoupper($module->getRequest()->extension)) {
			
			case 'DBG':
				echo '<pre>';
				print_r($module->getList());
				break;
			
			case 'JSON':
				$presentation = new JSONPresentation($module, $data);
				break;
				
			case 'XML':
				$presentation = new XMLPresentation($module, $data);
				break;

			/* 
			 * RAW - We remove the template, and display without the template
			 * Don't break this - as we want to hit next case as well
			 */
			case 'RAW':
			case 'RSS':				
				$module->getDocument()->template = '';
							
			case 'HTML':
			default:
				// Mark: Get a fresh action, it can have changed..
				if (file_exists($module->getModuleInfo()->getModulePath() . self::MODULE_FOLDER_HTML . $module->getRequest()->action . '.php')) {
					$presentation = new TemplatePresentation($module);
				}
								
				break;				
		}
			
		
		// If we have a valid presentation, we use it, and display content...
		if (!is_null($presentation)) {
			$presentation->display();
			return;
		}
		
		// Display raw data
		echo $data;
		
		return;
	}
	
}

?>