<?php

/**
 * 
 * Handeling the Page Request
 * 
 * @author Glenn O Larsen <glenn.larsen@openit.com>
 * 
 * @package mvc
 * @version v3.0
 * 
 */

class Request {
	
	
	/**
	 * This variable helds how many subs that should be poped of the uri string
	 * 
	 * This is used when we have sub-modules, so we return the correct module, with the correct reqest details..
	 *
	 * @var int
	 */
	static public $POPMODULE = 0;
	
	
	static private $extension = null; 
	
	
	static public function getActions() {
		$uri = self::getBaseUri();

		// Do we have an extension?
		if (strstr($uri, '.')) {
			self::$extension = substr($uri, strpos($uri, '.')+1);
			$uri = substr($uri, 0, strpos($uri, '.'));
		}		
		return strstr($uri, '/') ? explode('/', $uri) : array($uri); 			
	}
	
	static public function getExtension() {
		return self::$extension;
	}
		
	
	/**
	 * Return the ID that was given in the URI
	 * 
	 * E.g.: /helper/users/65/edit - Would give to 65 as id
	 * Return null, if there is no id found in the URI
	 *  
	 * @return int Id from URI
	 */
	static public function getId() {
		$actions = self::getActions();
		if (isset($actions[1]) && is_numeric($actions[1])) {
			return $actions[1]; 
		}
		return null;
	}

	
		
	/**
	 * Return the request URI, without doubble slahses, and dotted pathes..
	 *
	 * @return String Request URI
	 */
	static private function getUri($withoutExtraAttributes = false) {
		
		// Make sure we have a request uri, should always be deliverd from php
		if (!isset($_SERVER['REQUEST_URI'])) throw new Exception('Can\'t get the request uri from php engine...');
		
		// Set to local variable	
		$uri = $_SERVER['REQUEST_URI'];
		
		// Remove doubble / in the whole uri
		while (strstr($uri, "//")) {
			$uri = str_replace("//", "/", $uri);
		}
		
		// Remove extra / at the start of the uri
		while (substr($uri, 0, 1) == "/") {
			$uri = substr($uri, 1);
		}

		// Remove helper file from beginning of file..
		// This should always be the first element in the string
		
		
		// We now pop of each modules before current module..
		// We add one, since this is our helper..
		for ($i=0; $i<self::$POPMODULE+1; $i++) {
			$uri = substr($uri, strpos($uri, '/')+1);				
		}
				
		// Remove extra / at the start of the uri
		// We run this again, in case the URI is something like: //helper///////tooomanyslahses//
		while (substr($uri, 0, 1) == "/") {
			$uri = substr($uri, 1);
		}
		
		// Remove all ? - and keep the string before ?
		if ($withoutExtraAttributes) {
			while (strpos($uri, "?")) {
				$uri = substr($uri, 0, strpos($uri, "?"));
			}
		}
				
		// Remove / at the end of the uri
		while (substr($uri, -1) == "/") {
			$uri = substr($uri, 0, -1);
		}
				
		return $uri;		
	}
	
	
	/**
	 * Return base URI
	 * 
	 * Return the URI without extra paramateres, like ?hello=world
	 *
	 * @return String Request URI
	 */
	static public function getBaseUri() {
		return self::getUri(true);
	}
	
	
	static public function check() {
		$uri = $_SERVER['REQUEST_URI'];
		while (strpos($uri, "?")) $uri = substr($uri, 0, strpos($uri, "?"));

		if (substr($uri, -1) == '/') {
			
			$newUri = $_SERVER['REQUEST_URI'];
			$newParams = '';
			while (strpos($newUri, "?")) {
				$newParams .= substr($newUri, strpos($newUri, "?"));
				$newUri = substr($newUri, 0, strpos($newUri, "?"));
			}
			
			// Remove / at the end of the uri
			while (substr($newUri, -1) == "/") {
				$newUri = substr($newUri, 0, -1);
			}
						
			header('Location: ' . $newUri.$newParams);
			exit;
		}		
	}
		
	
}


?>