<?php

class RequestHandler {
	
	const WRAPPER = 'module';
	const MODULE_DIR = '../modules/'; // relative path from the helper file..!
	
	static private $currentPath;	
	
	static public $aliasList;

	static public function start($parentActions = null) {
			
		// Get acctions
		$actions = Request::getActions();
		
		// If URL is wrong constructed, it will redirect..
		//Request::check();
				
		// Get module
		$module = self::getModule();
		$id = Request::getId();
		
		// Check if we should send parent actions
		if (is_array($parentActions) && method_exists($module, 'setParentActions')) {
			$module->setParentActions($parentActions);
		}
		
			
		// Get main action command
		$action = is_null($id) ? (isset($actions[1]) ? $actions[1] : 'index') : (isset($actions[2]) ? $actions[2] : 'show');
		
		// Check if its a sub module
		if (method_exists($module, 'getSubModules')) {
			foreach ($module->getSubModules() as $submodule) {
				if (strtolower($submodule) == $action) {
					Request::$POPMODULE++;
					if (is_numeric($id)) Request::$POPMODULE++;
					RequestHandler::start($actions);
					return;
				}
			}
		}
		
		// If we cant find the action, we use the default one...
		if (!method_exists($module, $action)) {
			$id = $action;
			$action = 'show';
			if (!method_exists($module, $action)) {
				throw new Exception('Can\'t find action "' . $action . '" methode for module ' . $module->getClassName());
			}
		} 
		
			
		// We get the data as a string, since we might want to handle this in a diffrent way..
		ob_start();
		$module->$action($id);
		$data = ob_get_contents();
		ob_end_clean();
		
		
		// Display data based on extension
		switch (strtolower(Request::getExtension())) {
			case 'json':
				header("Status: 200");
				header('Content-type: application/x-json');
				if (!is_null($module->getList())) {
					$list = array();						
					foreach ($module->getList() as $model) {
						$mod = new ClassModifyer($model);
						$list[] = $mod->getValues();
					}
					echo json_encode($list);
				} else if (is_null($module->getModel())) {
					echo $data;
				} else {
					$mod = new ClassModifyer($module->getModel());
					echo json_encode($mod->getValues());
				}
				break;
				
			case 'xml':
				header("Status: 200");
				header('Content-type: text/xml');
				if (is_null($module->getModel())) {
					$list = array();						
					foreach ($module->getList() as $model) {
						$mod = new ClassModifyer($model);
						$list[] = $mod->getValues();
					}
					echo XMLSerializer::getXML(get_class($model), $list, true);
				} else {
					$mod = new ClassModifyer($module->getModel());
					echo XMLSerializer::getXML(get_class($module->getModel()), $mod->getValues());
				}
				break;
				
			case 'raw':		
				header("Status: 200");
				//header('Content-type: application/x-json');
				echo $data;
				break;
		
			default:
				header("Status: 200");
				header('Content-type: text/html');
				
				// Display framework				
				if (file_exists(self::$currentPath . 'html/' . $action . '.php')) {
					
					$controller = $module;
					
//					$uri = $module->getClassName();
					$model = $module->getModel();
					$list = $module->getList();

					// TODO: Replace with real template system
					ob_start();
					include self::$currentPath . 'html/' . $action . '.php';
					$data .= ob_get_contents();
					ob_end_clean();
					
					// TODO: Replace with real template system
					ob_start();
					include self::MODULE_DIR . 'html/' . $module->getDocument()->template . '.php';
					//include self::MODULE_DIR . 'html/normal.php';					
					$html = ob_get_contents();
					ob_end_clean();
					$html = str_replace('#TITLE#', $module->getDocument()->title, $html);
					$html = str_replace('#MESSAGE#', $module->getDocument()->message->text, $html);
					echo str_replace('#CONTENT#', $data, $html);					
					echo '<pre>'; print_r($module->getDocument());
					
				} else {
					echo 'NO HTML ' . self::$currentPath . 'html/' . $action . '.php';
					echo $data;
					var_dump($module->getModel());
					var_dump($module->getList());
				}
				break;
		}
					
	}
	
	static public function getModule() {
		$actions = Request::getActions();
		
		// Check that we have a module name in the uri
		if (!isset($actions[0])) throw new Exception('No module name in URI');
		
		// Local variable
		$module = strtolower($actions[0]);

		// Collect all aliases from modules
		$aliases = glob(self::MODULE_DIR . '*/alias.php');
		self::$aliasList = array();

		// Inner function for our alias files..
		if (!function_exists('registerAlias')) {
			function registerAlias($alias, $class) {
				RequestHandler::$aliasList[$alias] = $class;			
			}
		}
		
		// Let each file register their alias needs..
		foreach ($aliases as $alias) include $alias;
				
		// Check if current module name is in alias list..
		if (isset(self::$aliasList[$module])) {
			$module = self::$aliasList[$module];
		}
		
		// Check if we have a module with that name
		$classPath = self::MODULE_DIR . $module . '/classes/*.php';

		// Find controller
		$controllerFile = null;
		$files = glob($classPath);
		foreach ($files as $file) {
			if (strtolower(basename($file, '.php')) == $module) {
				self::$currentPath = realpath(dirname($file) . '/../') . '/';
				$controllerFile = $file;
				$class = basename($file, '.php'); 
				break;
			}
		}
		
		if (is_null($controllerFile)) throw new Exception('Did not find any module mathing request ' .  $module);
		
		// ADD PATH TO AUTOLOADER
		require_once $controllerFile;
			
		try {
			// Return an instance of the class
			return new $class;
		} catch (Exception $e) {
			return null;
		}
	}
	
	
	
	
}

?>