<?php

/**
 * This classes is used for easy validation of a request,
 * where we want to verify that the content entered is correct.
 * 
 * @author Glenn O Larsen <glenn.larsen@openit.com>
 */
class Validator {
	
	/**
	 * Local variable that holds the request object
	 *
	 * @var Request
	 */
	private $request;
	
	/**
	 * Local array that holds the error messages from the validation
	 *
	 * The array will be created like this:
	 *  $errors['fieldname'] = 'error message';
	 * 
	 * @var Mixed
	 */
	private $errors;
	

	
	public function __construct(&$request) {
		$this->request = &$request;
		$this->errors = array();
	}
	
	
	private function addRegularExpression($field, $expression, $error = null) {

		// Get field value
		$value = isset($this->request[$field]) ? $this->request[$field] : null;
		
		// If null or empty, we skip it..
		// Use requerd to make sure that the field is sent.. 
		if (is_null($value) || empty($value)) return true;

		// Test with regular expression
		if (preg_match($expression, $value)) return true;

		// If it's not valid.. 
		$this->errors[$field] = $error;
		return false;
	}
	
	
	
	

	/**
	 * The value is restricted to valid email
	 *
	 * @param String $mail Variable to check
	 * @param String $error Error message if this test fails
	 * @ignore NOT IMPLEMENTED YET
	 */
	public function isEmail($mail, $error = null) {
		
	}
	
	/**
	 * Check if expression equals to true
	 *
	 * @param Mixed $boolean
	 * @param String $error Error message if this test fails
	 * @ignore NOT IMPLEMENTED YET
	 */
	public function isTrue($boolean, $error = null) {
		
	}
	
	/**
	 * Check if two variables are equal to each other
	 *
	 * @param String $param1 Param one
	 * @param String $param2 Param two 
	 * @param String $error Error message if this test fails
	 * @ignore NOT IMPLEMENTED YET
	 */
	public function isEqual($param1, $param2, $error = null) {
		
	}
	
	/**
	 * The value is restricted to alphabetic chars
	 *
	 * @param String $alpha Variable to check
	 * @param String $error Error message if this test fails
	 * @ignore NOT IMPLEMENTED YET
	 */
	public function isAlpha ($alpha, $error = null) {
		
	}
	
	/**
	 * The value is restricted to alphanumeric characters only.
	 * 
	 * Expression: a-z, 0-9, space, and punkt(.) is accepted
	 *
	 * @param String $alphanum Variable to check
	 * @param String $error Error message if this test fails
	 */
	public function isAlphanum ($alphanum, $error = null) {
		return $this->addRegularExpression($alphanum, "/^[\040a-z0-9\.]*$/i", $error);
	}
	
	/**
	 * The field doesn’t accept digit chars.
	 *
	 * @param String $nodigit Variable to check
	 * @param String $error Error message if this test fails
	 * @ignore NOT IMPLEMENTED YET
	 */
	public function isNoDigit ($nodigit, $error = null) {
		
	}
	
	/**
	 * The value is restricted to digit (no floating point number) chars
	 *
	 * @param String $digit Variable to check
	 * @param String $error Error message if this test fails
	 * @ignore NOT IMPLEMENTED YET
	 */
	public function isDigit ($digit, $error = null) {
		
	}
	
	/**
	 * The value is restricted to number, including floating point number.
	 *
	 * @param String $number Variable to check
	 * @param String $error Error message if this test fails
	 * @ignore NOT IMPLEMENTED YET
	 */
	public function isNumber ($number, $error = null) {
		
	}
	
	/**
	 * The value is restricted to phone chars.
	 *
	 * @param String $phone Variable to check
	 * @param String $error Error message if this test fails
	 * @ignore NOT IMPLEMENTED YET
	 */
	public function isPhone ($phone, $error = null) {
		
	}
	
	/**
	 * The value is restricted to url.
	 *
	 * @param String $url Variable to check
	 * @param String $error Error message if this test fails
	 * @ignore NOT IMPLEMENTED YET
	 */
	public function isUrl ($url, $error = null) {
		
	}
	
	/**
	 * The field becomes required. Can't be null or not containing any value.
	 *
	 * @param String $param Variable to check
	 * @param String $error Error message if this test fails
	 * @ignore NOT IMPLEMENTED YET
	 */
	public function setRequired($param, $error = null) {
		if (isset($this->request[$param]) && !empty($this->request[$param])) {
			return true;
		}
		
		$this->errors[$param] = $error;
		return false;
	}
	
	/**
	 * Validate the request
	 *
	 * @return boolean
	 */
	public function validate() {
		if (count($this->errors) == 0) return true;
		return false;
	}
	
	/**
	 * Returns the array with error messages on the fields that failed
	 *
	 * @return Array
	 */
	public function getErrors() {
		return $this->errors;	
	}
	
}

?>