<?php

/**
 * This class is used to decive which module to use
 * 
 * When there is no filters defined, it scan in this sequence, from the beginning of the uri:
 *  - /<loader>/{modulen-name}/{sub-module}	* N
 *  - /<loader>/{modulen-name}/{action}
 *  - /<loader>/{modulen-name}/{id}/{sub-module}
 *  - /<loader>/{modulen-name}/{id}/{action}
 *  - /<loader>/{modulen-name}
 * 
 * @static 
 */
class RequestHelper {
	
	const WRAPPER = 'module';
	const MODULE_DIR = '../modules/'; // relative path from the helper file..!
		
	static private $baseUri;
	static private $pop;
	static public $moduleList; // Must be public because of alias.php files
	
	static private $debug = 0;
	
	/**
	 * Get module based on request
	 * @return AbstractController
	 */
	static public function getModule(&$action = null) {
		
		self::$debug = SystemSettings::get(SystemSettings::SESSION_DEBUG);
		
		// Get the base uri
		$uri = RequestParser::getBaseUri();

		// Get an array with all the parts from base uri 
		$parts = strstr($uri, '/') ? explode('/', $uri) : array($uri);
				
		if (!isset($parts[0])) throw new Exception('No module name in URI');
		
		// Local variable to hold module
		/* @var $currentModule AbstractController */
		$currentModule = null;
		$moduleFolder = null;		
		$action = null;
					
		// Try to get as deep as possible..
		do {
			
			// Shift first element of parts array
			$current = array_shift($parts);
			$next = isset($parts[0]) ? $parts[0] : null;
						
			// Check if its a module, or a valid sub-module
			if (self::isModule($current) && (is_null($currentModule) || in_array($current, $currentModule->getSubModules()))) {			
				if (self::$debug) echo "\n" . $current . ' is a module..<br>';
				$currentModule = self::loadModule($current, new Request2(RequestParser::getUri(), RequestParser::getExtension(), $parts, null));
				$action = null; 
				continue;
			}
			
			// Check action
			if (method_exists($currentModule, $current)) {
				if (self::$debug) echo "\n" . $current . ' is an action of ' . $currentModule->getClassName() . '<br>';
				$action = $current;
				$currentModule->getRequest()->action = $current;
				continue;
			}
						
			// Check id and sub-module
			/*
			if (!is_null($next) && self::isModule($next) && in_array($next, $currentModule->getSubModules())) {
				if (self::$debug) echo "\n" . $next . ' is a sub module and ' . $current . ' is the id....<br>';
				array_shift($parts);
				//$currentModule = self::loadModule($next);
				$currentModule = self::loadModule($next, new Request2(RequestParser::getUri(), RequestParser::getExtension(), $parts, $current));
				$action = null;
				continue;
			}
			*/

			if (!is_null($next) && self::isModule($next) && in_array($next, $currentModule->getSubModules())) {
				if (self::$debug) echo "\n" . $next . ' is a sub module....<br>';
				array_shift($parts);
				//$currentModule = self::loadModule($next);
				$currentModule = self::loadModule($next, new Request2(RequestParser::getUri(), RequestParser::getExtension(), $parts, null));
				$action = null;
				continue;
			}			
			
			// Check id and action
			if (!is_null($next) && method_exists($currentModule, $next) && is_null($currentModule->getRequest()->id)) {
				if (self::$debug) echo "\n{$current} is an id, and $next is a action in the class " . $currentModule->getClassName() . '<br>';
				$action = $next;
				$currentModule->getRequest()->id = $current;
				$currentModule->getRequest()->action = $next;
				array_shift($parts);
				continue;
			}
			
			// If POST and no action, and no current or next
			if (
					!is_null($currentModule)
				&&	is_null($current)
				&& 	is_null($next)
				&& 	is_null($currentModule->getRequest()->id) 
				&& 	(is_null($currentModule->getRequest()->action) || $currentModule->getRequest()->action == 'create')
				&& 	(count($_POST) >= 1)
				&& 	method_exists($currentModule, 'put')
				) {
				$action = 'put';
				if (self::$debug) echo "\nWill call PUT action in " . $currentModule->getClassName() . '<br>';
				$currentModule->getRequest()->action = 'put';
				break;
			}
						
			// Check id with post request/action, and current but no next
			if (!is_null($currentModule) && !is_null($current) && is_null($next) && is_null($currentModule->getRequest()->id) && (count($_POST) >= 1) && method_exists($currentModule, 'update')) {
				$action = 'update';
				if (self::$debug) echo "\nWill call UPDATE action in " . $currentModule->getClassName() . '<br>';
				$currentModule->getRequest()->id = $current;
				$currentModule->getRequest()->action = 'update';
				break;
			}
			
			// Check id with default action
			if (!is_null($current) && method_exists($currentModule, 'show') && is_null($currentModule->getRequest()->id)) {
				$action = 'show';
				if (self::$debug) echo "\n{$current} will be inserted as an id to SHOW action in " . $currentModule->getClassName() . '<br>';
				$currentModule->getRequest()->id = $current;
				$currentModule->getRequest()->action = 'show';
				break;
			}
		
			break;
		} while(true);
		
		// Before we return, we try to controll check the action
		if ($currentModule instanceof AbstractController && is_null($action)) {

			// Do we have an id
			if (!empty($currentModule->getRequest()->id) && method_exists($currentModule, 'show')) {
				$action = 'show';
				$currentModule->getRequest()->action = 'show';
			}
			
			// No id
			if (empty($currentModule->getRequest()->id) && method_exists($currentModule, 'index')) {
				$action = 'index';
				$currentModule->getRequest()->action = 'index';
			}
			
		}
		
		
		if ($currentModule instanceof AbstractController) {
			return $currentModule;
		}
		
		print_r($currentModule);
		die('nothing..');
		
	}
	
	static private function isModule($name) {		
		// Load module list, if its empty
		if (is_null(self::$moduleList)) self::loadModuleList();		
		return isset(self::$moduleList[strtolower($name)]) ? true : false;
	}
	
	static private function loadModuleList() {
		
		self::$moduleList = array();
		
		// Collect real names
		$moduleClasses = glob(self::MODULE_DIR . '*/classes/*.php');
		foreach ($moduleClasses as $moduleClass) {
			
			$moduleName = basename($moduleClass, '.php');
			$moduleFolder = str_replace(self::MODULE_DIR, '', dirname(dirname($moduleClass)));
			
			if (strtolower($moduleName) == strtolower($moduleFolder)) {
				self::$moduleList[strtolower($moduleName)] = $moduleName;
			}
		}
		
		// Load aliases		
		$aliases = glob(self::MODULE_DIR . '*/alias.php');
		
		// Inner function to support aliase files..
		if (!function_exists('registerAlias')) {
			function registerAlias($alias, $class) {
				$alias = strtolower($alias);
				$class = strtolower($class);
				if (isset(RequestHelper::$moduleList[$class])) {
					RequestHelper::$moduleList[$alias] = RequestHelper::$moduleList[$class];	
				}
			}
		}
		
		// Load each alias file
		foreach ($aliases as $alias) include $alias;						
	}
	
	static private function loadModule($moduleName, Request2 $request) {
		
		// Let Autloader know about our classes..
		AutoLoader::addFolder(self::MODULE_DIR);

		// Get real classname
		$moduleName = self::$moduleList[$moduleName];
		
		// Return new instance of class 
		return new $moduleName($request);
	}

}
?>