<?php

class MappingDatabaseConnector {
	
	private static $dbh;
	
	static private function getConnection() {
		if (is_null(MappingDatabaseConnector::$dbh)) {
			try {
				//if the mapping file does not exist try creating it
				if (file_exists(realpath($_SERVER['DOCUMENT_ROOT'] . '/../db/map/map.dbl'))){
					self::$dbh = new PDO('sqlite:' .realpath($_SERVER['DOCUMENT_ROOT'] . '/../db/map/map.dbl'));
					self::$dbh->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
				}
			} catch (PDOException $e) {
				die($e->getMessage() . 'Error no: 87263723');
			}
		}
		return self::$dbh;
	}

	
	static public function set($table, $id, $fields, $object) {
		
		$dbh = self::getConnection();
		
		// Verify that we have an object		
		if (!is_object($object) || !method_exists($object, 'get' . $id)) {
			die('Error: 2873283');
		}
		
		// Get the id of the object
		$uid = $object->{'get' . $id}();
		
		// Decide if we have a new or existing object
		$isNew = is_null($uid) || !is_numeric($uid) ? true : false;
		
		$values = array();
		
		// Collect SQL data
		foreach ($fields as $key) {
			if (!method_exists($object, 'get' . $key)) die('Error: 89826732 ');
			$value = $object->{'get' . $key}();
			$values[$key] = $value;
		}

		
		// Build SQL string
		$SQL = '';

		// Insert statement
		if ($isNew) {
			$fields = implode(',', array_keys($values));
			$SQL = "INSERT INTO {$table} ({$fields}) VALUES (";
			$first = true;
			foreach ($values as $key => $value) {
				if ($key == $id) {
					$SQL .= ($first ? '' : ', ') . 'null';
					$first = false;
					continue;					
				}
				$SQL .= ($first ? '' : ', ') . '"' . stripslashes($value) . '"';
				$first = false;
			}
			$SQL .= ");";
		}
		
		// Update statement
		else {
			$SQL = "UPDATE {$table} SET ";
			$first = true;
			foreach ($values as $key => $value) {
				if ($key == $id) {
					$SQL_WHERE = " WHERE {$key} = {$value}";
					continue;
				}
				if($key == "name" && $value == "uid")continue;
				$SQL .= ($first ? '' : ', ') . "{$key} = " . "'" . stripslashes($value) . "'";
				$first = false;
			}
			
			if (!isset($SQL_WHERE)) die('Error: 892832');
			$SQL .= $SQL_WHERE . ";";			
		}
		Logger::Debug('MappingDatabaseConnector', "Executing " . $SQL);
		try {
//			Logger::Debug('GLENN', $SQL);
			$result = self::$dbh->query($SQL);
			$rows = $result->fetchAll(PDO::FETCH_ASSOC);

			// If new, return insert id
			if ($isNew) {
				return $dbh->lastInsertId(); 				
			}
			
		} catch (PDOException $e) {
			die($e->getMessage() . ' - ' . $SQL);
			//Logger::Critical(__CLASS__ . '::' . __METHOD__, 'Database error: ' . $e->getMessage(), __LINE__);;
			return false;
		}
		
		return true;
		
	}
	
	static public function delete($table, $id, $uid) {
		
		$dbh = self::getConnection();
		
		// Verify that we have an object		
		if (!is_numeric($uid)) {
			die('Error: 232421342314');
		}
		
		// Decide if we have a new or existing object
		$isNew = is_null($uid) || !is_numeric($uid) ? true : false;

		// Build SQL string
		$SQL = "DELETE FROM {$table} WHERE {$id} = {$uid};";
		
		try {		
			$result = self::$dbh->query($SQL);
			$rows = $result->fetchAll(PDO::FETCH_ASSOC);

			// If new, return insert id
			if ($isNew) {
				return $dbh->lastInsertId(); 				
			}
			
		} catch (PDOException $e) {
			Logger::Critical(__CLASS__ . '::' . __METHOD__, 'Database error: ' . $e->getMessage(), __LINE__);;
			return false;
		}
		
		return true;
		
	}
	
	
	static public function get($table, $fields, $class, $ids = null, $asList = false) {
		
		$dbh = self::getConnection();
		
		try {
			$SQL = "SELECT * FROM $table WHERE 1=1 "; // 1=1 since I used AND in next loop - yes, I'm lacy
			
			if (!is_null($ids)) {
				foreach ($ids as $idQuery) $SQL .= " AND {$idQuery}"; 
			}
			
			$result = self::$dbh->query($SQL);
			$rows = $result->fetchAll(PDO::FETCH_ASSOC); 
		} catch (PDOException $e) {
			Logger::Critical(__CLASS__ . '::' . __METHOD__, 'Database error: ' . $e->getMessage(), __LINE__);;
			return false;
		}
		
		// If not class is defined, return the rows
		if ($class == null) return $rows;
		
		// If there is no results returned from the query, return null
		if (count($rows) == 0) return null;

		
		if (count($rows) == 1 && !$asList) {
			$object = new $class;
			$row = $rows[0];
			foreach ($fields as $index => $field) {
				if (isset($row[$field])) {
					$object->{'set' . $field}($row[$field]);
				}
			}
			return $object;
		} else {
			$objects = array();						
			foreach ($rows as $row) {
				$object = new $class;				
				foreach ($fields as $index => $field) {
					if (isset($row[$field])) {
						$object->{'set' . $field}($row[$field]);
					}
				}
				$objects[] = $object;
			}
			return $objects;
		}
			
		return false;
	}
	
	
	static public function getAll($table, $fields, $class) {
		
		$dbh = self::getConnection();
		
		try {
			$result = self::$dbh->query("SELECT * FROM $table");
			$rows = $result->fetchAll(PDO::FETCH_ASSOC); 
		} catch (PDOException $e) {
			//die($e->getMessage());
			return false;
		}
		
		if (count($rows) != 0) {			
			$objects = array();						
			foreach ($rows as $row) {
				$object = new $class;				
				foreach ($fields as $index => $field) {
					if (isset($row[$field])) {
						$object->{'set' . $field}($row[$field]);
					}
				}
				$objects[] = $object;
			}
			return $objects;
		}	
		return false;
	}
	
	static public function query($SQL) {
		
		$dbh = self::getConnection();
		
		try {
			$result = self::$dbh->query($SQL);
			$rows = $result->fetchAll(PDO::FETCH_ASSOC); 
		} catch (PDOException $e) {
			die('Error: 27387238');
			return false;
		}
		
		return $rows;
	}
	
}

?>
