<?php

class Parser {

	private $iterator;
	private $location;
	

	private $config;
	
	public function __construct($data, $config = null) {

		$this->location = -1;
		$this->config = $config;
		
		if (!is_array($data)) $data = explode("\n", $data);

		// Check if we should skip blank/comment lines
		if ($config != null && (
				$config->get(ParserSettings::SKIP_BLANK_LINES) ||
				$config->get(ParserSettings::ONLY_LINES_THAT_STARTS_WITH) ||
				$config->get(ParserSettings::SKIP_COMMENTS))) {				
			$this->iterator = array();
			foreach ($data as $line) {
				if ($config->get(ParserSettings::SKIP_BLANK_LINES) && trim($line) == "") continue;
				if ($config->get(ParserSettings::SKIP_COMMENTS) && substr(ltrim($line),0,1) == $config->get(ParserSettings::COMMENT_CHAR)) continue;
				if ($config->get(ParserSettings::ONLY_LINES_THAT_STARTS_WITH)) {
					$start = $config->get(ParserSettings::ONLY_LINES_THAT_STARTS_WITH);
					if (substr(trim($line), 0, strlen($start)) != $start) continue;
				}
							
				$this->iterator[] = $line;				
			}
		} else {
			$this->iterator = $data;
		}
				
	}
	
	public function hasMore() {		
		return (count($this->iterator)-1) > $this->location;
	}
	
	/**
	 * @return ParserLine
	 */
	public function next($config = null) {		
		$this->location++;
		return new ParserLine($this->iterator[$this->location], (!is_null($config)) ? $config : $this->config);
	}
	
	
		
}

?>