<?php

class ParserLine {
	
	private $data;
	private $dataSplitted;
	private $location;
	private $config;
	
	
	public function __construct($data = null, $config = null) {
		$this->location = -1;		
		$this->data = $data;
		$this->config = is_null($config) ? new ParserSettings() : $config;
		$this->splitData();
	}
	
	
	public function __toString() {
		return $this->data;
	}
	
	public function toString() {
		return $this->data;
	}
	
	public function hasMore() {
		return (count($this->dataSplitted)-1) > $this->location;
	}
		
	
	/**
	 * @return ParserLine
	 */
	public function next($config = null) {
		$this->location++;

		if ($this->config->get(ParserSettings::SPLIT) == null) {
			return new ParserLine($this->data, $config);
		}		
		
		if (!isset($this->dataSplitted[$this->location])) return null;

		return new ParserLine($this->dataSplitted[$this->location], $config); 
	}
	
	/**
	 * @return ParserLine
	 */	
	public function get($int, $config = null) {
		if (!isset($this->dataSplitted[$int])) return new ParserLine();
		return new ParserLine($this->dataSplitted[$int], $config);
	}
	
	
	public function count() {
		return count($this->dataSplitted);
	}
	
	private function splitData() {

		switch ($this->config->get(ParserSettings::SPLIT)) {
			
			case ParserSettings::SPLIT_TYPE_WHITE_SPACE:
				if (strstr(PHP_OS, "WIN")){
				$this->dataSplitted = preg_replace('/\s+/', '+', $this->data);
				$this->dataSplitted = preg_replace('/\+/', ' ', $this->dataSplitted, 1);
				
				$this->dataSplitted = explode(' ', $this->dataSplitted);
				$this->dataSplitted = preg_replace('/\+/', ' ', $this->dataSplitted);
				} else {
				$this->dataSplitted = preg_replace('/\s+/', ' ', $this->data);
				$this->dataSplitted = explode(' ', $this->dataSplitted);				
				}
				break;
				
			case ParserSettings::SPLIT_TYPE_SPACE:
				$this->dataSplitted = explode(' ', $this->data);
				break;
				
			case ParserSettings::SPLIT_TYPE_COMMA:
				$this->dataSplitted = explode(',', $this->data);
				break;
				
			case ParserSettings::SPLIT_TYPE_PIPE:
				$this->dataSplitted = explode('|', $this->data);
				break;
				
			case ParserSettings::SPLIT_TYPE_TAB:
				$this->dataSplitted = explode("\t", $this->data);
				break;

			case ParserSettings::SPLIT_TYPE_EQUALS:
				$this->dataSplitted = explode('=', $this->data);
				break;

			case ParserSettings::SPLIT_TYPE_SEMI_COLON:
				$this->dataSplitted = explode(':', $this->data);
				break;
				
			case ParserSettings::SPLIT_TYPE_PROSENTAGE:
				$this->dataSplitted = explode('%', $this->data);
				break;

			case ParserSettings::SPLIT_TYPE_PARENTESE_START:
				$this->dataSplitted = explode('(', $this->data);
				break;
				
			default:
				if ($this->config->get(ParserSettings::SPLIT) != '') {
					$this->dataSplitted = explode($this->config->get(ParserSettings::SPLIT), $this->data);
				}
				break;
				
		}
				
	}
			
}

?>