<?php

// Load the initiation file
require_once($_SERVER["DOCUMENT_ROOT"] . "/../classes/initiation.php");

// Load presentation layer class
$presentation = new Presentation();

// Load template
$reportSelection = new ReportSelection();
$templateLoader = new TemplateLoader();

/*@var $template Template */
$template = $templateLoader->getTemplate($reportSelection->get("PRESENTATION_ID"));


// If we miss a template - display an error
if (!$template) {
	$presentation->display("pages/template_selection/error_selections.php");
	return;
}

// Check if we miss a user input
if (!$template->isSelectionValid()) {
	header("Location: input.php?showMissing=true");
	return;
}

// If there is no template selected, show error message
if ($reportSelection->get("PRESENTATION_ID") == "") {
	$presentation->display("pages/template_selection/error_selections.php");
	return;
}

// Check if a folder is set, if not, set the default to be the same as the "config"
if ($reportSelection->get("generation_folder") == "") {
	$reportSelection->set("generation_folder", trim($template->getFolder()));
}

// Set value from template to this instance
if ($reportSelection->get("generation_period") == "") {
	$reportSelection->set("generation_period", (isset($template->reportData["period"]) ? $template->reportData["period"] : ""));
}

// Set default values
if ($reportSelection->get("generation_timeout") == "") $reportSelection->set("generation_timeout", (isset($template->reportData["timeout"]) ? $template->reportData["timeout"] : "10"));
if ($reportSelection->get("generation_periodic") == "") $reportSelection->set("generation_periodic", "D");
if ($reportSelection->get("generation_periodic_value") == "") $reportSelection->set("generation_periodic_value", "1");


// Check if we are using resolution
if (!$reportSelection->get("resolution") && isset($template->reportData["resolution"]) && $template->reportData["resolution"] != "") {
	switch ($template->reportData["resolution"]) {
		case 300: 		$value = "0"; break;
		case 900: 		$value = "1"; break;
		case 3600: 		$value = "2"; break;
		case 86400: 	$value = "3"; break;
		case 604800:	$value = "4"; break;
		case "M": 		$value = "5"; break;
		case "Y": 		$value = "6"; break;
		default: return;
	}
	$reportSelection->set("resolution", 1);
	$reportSelection->set("generation_resolution", $value);
}


// Send data to the presentation displayer
$presentation->assign("selection", $reportSelection);
$presentation->assign("inputDisabled", !$template->isUserInputEnabled());
$presentation->assign("folders", $_SESSION["USER"]->getFolders());

// Display the presentation
$presentation->display("pages/template_selection/generate.php");

?>