<?php

class Mapping_Source extends AbstractController {
	
	private $mappingSource;
	private $mappingSources;
	
	public function getSubModules() {
		return array('mapping_source_settings', 'settings');
	}
	
	public function getModel() {
		return $this->mappingSource;
	}
		
	public function getList() {
		return $this->mappingSources;
	}
		
	
	public function index() {
		$this->mappingSources = MapppingManager::getSources();
	}
	
	public function init() {
		// Load the initiation file
		require_once($_SERVER["DOCUMENT_ROOT"] . "/../classes/initiation.php");
	}
	
	public function create() {
		
		$this->index();
		
		$this->getRequest()->action = 'index';
		
		$this->mappingSource = new MappingSource();
		
		// Set fields
		//$this->getDocument()->fields = $this->render('fields');
		
		// Set config attributes
		//$this->getDocument()->config = $this->render('config_ad');
		
		// Use index as default template, as longs as we dont want to display raw
		$this->getDocument()->edit = $this->render('edit');			
		
	}
	
	public function put() {
		$source = new MappingSource(null, $this->getRequest()->_POST['typeid'], $this->getRequest()->_POST['name']);
		$status = MappingSource::store($source);
		return $this->redirect(array($status, 'edit'));
	}
	
	
	public function show() {
		return $this->redirect(array(null, null, 'edit'));
	}

	
	public function getMenu() {
		return 'admin';
	}
	
	public function getSubMenu() {
		return 'mappingsub';
	}
	
	public function edit() {
		
		$this->index();
		
		if (!is_numeric($this->getRequest()->id)) {
			return $this->redirect(array(null));
		}
		
		$this->mappingSource = MappingSource::getSource($this->getRequest()->id);
		
		// Set fields
		$this->fields();
		$this->getDocument()->fields = $this->render('fields');
		
		// Set config attributes
		switch ($this->mappingSource->getTypeId()) {
			
			case MappingSourceType::AD:
			case MappingSourceType::LDAP:
				$this->getDocument()->config = $this->render('config_ad');
				break;
			
			case MappingSourceType::TEXT:
				$this->getDocument()->config = $this->render('config_file');
				break;
		}
		
		// Use index as default template, as longs as we dont want to display raw
		if ($this->getRequest()->extension != 'raw') {
			$this->getRequest()->action = 'index';
			$this->getDocument()->edit = $this->render('edit');			
		}
		
	}
	
	public function validateOFF() {
		
		$this->getValidator()->setRequired('name', 'Need a name');
		$this->getValidator()->setRequired('typeid', 'Need a valid type');

		if ($this->getRequest()->id != '') {
			
			switch (MappingSource::getSource($this->getRequest()->id)->getTypeId()) {
				
				case MappingSourceType::AD:
				case MappingSourceType::LDAP:
					$this->getValidator()->setRequired('server', 'Need a Server');
					$this->getValidator()->setRequired('port', 'Need a Server Port');
					$this->getValidator()->setRequired('authid', 'You need to set a username');
					$this->getValidator()->setRequired('path', 'Need AD path');
					break;
				
				case MappingSourceType::TEXT:
					$this->getValidator()->setRequired('file', 'Need file');
					$this->getValidator()->setRequired('separator', '!!');
					break;
			}
			
		}		
	}
	
	
	
	public function update() {

		// Check for delete of source
		if (isset($_REQUEST['delete'])) {
			$id = $this->getRequest()->id;
			$sql = "SELECT * FROM mapConfig WHERE sourceId = $id ";
			$result = MappingSource::query($sql);
			
			if(count($result) <= 0){
				MappingSource::delete($id);
				return $this->redirect(array());
			}
			echo $id;
			return $this->redirect(array(null, null, 'edit'));
		}
		
		// Update the source
		$source = MappingSource::getSource($this->getRequest()->id);
		$source->setName($this->getRequest()->_POST['name']);
		
		$newType = $source->getTypeId() != $this->getRequest()->_POST['typeid'] ? true : false;
		
		$source->setTypeId($this->getRequest()->_POST['typeid']);
		
		$status = MappingSource::store($source);
			
		if (!$status) die('Error: 273872387');
		
		// If new type, we dont need the rest..
		if ($newType) {
			return $this->redirect(array(null, null, 'edit'));
		}

		
		// Update Source Config
		
		//var_dump($this->getRequest()->_POST); exit;
		
		// if AD
		switch ($this->getRequest()->_POST['typeid']) {
			
			// Handle AD and LDAP
			case MappingSourceType::AD:
			case MappingSourceType::LDAP:
				$source->setConfig('Server', $this->getRequest()->_POST['server']);
				$source->setConfig('Port', $this->getRequest()->_POST['port']);
				$source->setConfig('AuthID', $this->getRequest()->_POST['authid']);
				$source->setConfig('Path', $this->getRequest()->_POST['path']);			
				$source->setConfig('Schema', isset($this->getRequest()->_POST['schema']) ? $this->getRequest()->_POST['schema'] : "");
				if ($this->getRequest()->_POST['password'] != '********') {
                                        $b64box = base64_encode($this->getRequest()->_POST['password']);
					$source->setConfig('Password', base64_encode($b64box."GALUT="));
				}
				break;
				
			// Handle file
			case MappingSourceType::TEXT:
				$source->setConfig('File', $this->getRequest()->_POST['file']);
				$source->setConfig('Separator', $this->getRequest()->_POST['separator']);
				break;				

			default:
				var_dump($this->getRequest()->_POST);
				exit;			
		}
		
		// Store source fields
		if (isset($this->getRequest()->_POST['fields']) && is_array($this->getRequest()->_POST['fields'])) {
			$existingFields = array();
			
			// get all existing fields for the given source
			$existingFields = MappingSourceField::getFields($source->getId());
			
			// Check if an element has been managed
			$managed = array();

			// Check if there are existing fields and if it is an array
			if(count($existingFields) > 0 && !is_array($existingFields)){
				$existingFields = array($existingFields);
			}
			
			// Check for existing and deleted fields
			if(is_array($existingFields)){
				foreach ($existingFields as $existingField) {
					foreach ($this->getRequest()->_POST['fields'] as $sName => $sField) {
						
						// Continue if we dont find the field in the post
						if ($sName != $existingField->getName()) continue;
						
						// If include is set in the post, it exists in new post
						if (isset($sField['include'])) {
		
							$historyChanged = ($existingField->getHistory() == isset($sField['history']));
							$uniqChanged = (($this->getRequest()->_POST['fields_uid'] == $sName) == $existingField->getUid());
							
							
							// it has changed, set status to changed	
							if ($historyChanged || $uniqChanged) {		
								$existingField->setHistory(isset($sField['history']) ? 1 : 0);
								$existingField->setUid(($this->getRequest()->_POST['fields_uid'] == $sName ? 1 : 0));
								$existingField->update();	
							}				
						}
						
						// If include is not set, its removed
						else {						
							$existingField->delete();
						}

						$managed[$sName] = true;
						
						// Goto next existing field
						continue 2;
					}	
					$managed[$existingField->getName()] = true;
					$existingField->delete();					
				}
			}
			
			// Add new fields
			foreach ($this->getRequest()->_POST['fields'] as $sName => $sField) {				
				if (!isset($managed[$sName]) && isset($sField['include'])) {
					MappingSourceField::store(new MappingSourceField(null, $source->getId(), $sName, ($this->getRequest()->_POST['fields_uid'] == $sName ? '1' : '0'), (isset($sField['history']) ? '1' : '0')));
				}
			}
						
		}
				
		return $this->redirect(array(null, null, 'edit'));
	}
	
	
	public function config() {
		// Only support raw view..
		if ($this->getRequest()->extension != 'raw') exit;
		$this->mappingSource = MappingSource::getSource($this->getRequest()->id);
		echo $this->render('config_ad');
	}
	
	public function fields() {	
		// Only support raw view..
		//if ($this->getRequest()->extension != 'raw') return;
		$schemaFields = array();
		if ($this->getRequest()->id == '') return;
		
		$this->mappingSource = MappingSource::getSource($this->getRequest()->id);
		$schema = MappingSourceField::getFields($this->getRequest()->id);

		if (isset($this->getRequest()->_REQUEST['all'])) {
			$fields = MappingBackendConnector::getFields($this->getRequest()->id);
			if(count($schema) > 1){
				$schemaFields = MappingSourceField::getFields($this->getRequest()->id);
			}
			elseif(count($schema) == 1){
				$schemaFields[] = MappingSourceField::getFields($this->getRequest()->id);	
			}
					
			if ($this->mappingSource->getTypeId() == MappingSourceType::AD) {
				$fields[] = 'sAMAccountName'; // TODO: I should get this one from perl script - not hard coded like this..
			}
			foreach ($fields as $field) {
				foreach ($schemaFields as $schemaField) {
					if ($schemaField->getName() == $field) continue 2;
				}
				$schemaFields[] = new MappingSourceField(null, $this->getRequest()->id, $field);
			}
		}
		else {
			$schemaFields = MappingSourceField::getFields($this->getRequest()->id);
		}
		
		$this->getDocument()->schemaFields = $schemaFields;
		//$this->getRequest()->action = ''; var_dump($this->getDocument()->schemaFields);
	}
	
	public function connect() {
                $connectresponse = MappingBackendConnector::canConnect($this->getRequest()->id,$this->getRequest()->_POST);
                switch($connectresponse){
                    case 0: echo "Cannot connect to server"; break;
                    case 1: echo "Success"; break;
                    case 2: echo "Cannot create session"; break;
                    case 3: echo "Incorrect username/password"; break;
                }
	}
	
	public function classes() {
		echo json_encode(MappingBackendConnector::getSchemas($this->getRequest()->id));
	}
	
	
}


?>